/*
 * Floating-point operations using the underlying 'double' type.
 *
 * This file is meant to be included, not compiled by itself. It may
 * be included once or twice; the macro FPC will be defined only for the
 * second inclusion. At the first inclusion, the 'fpr' type and the
 * operator functions are defined; the second inclusion defines the
 * arrays of constants for the FFT.
 *
 * Functions are documented in 'internal.h'.
 *
 * ==========================(LICENSE BEGIN)============================
 *
 * Copyright (c) 2017  Falcon Project
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * ===========================(LICENSE END)=============================
 *
 * @author   Thomas Pornin <thomas.pornin@nccgroup.trust>
 */

#ifndef FPC

#include <math.h>

typedef struct { double v; } fpr;

static inline fpr
FPR(double v)
{
	fpr x;

	x.v = v;
	return x;
}

static inline fpr
fpr_of(int64_t i)
{
	return FPR((double)i);
}

static inline fpr
fpr_scaled(int64_t i, int sc)
{
	return FPR(ldexp((double)i, sc));
}

static inline fpr
fpr_inverse_of(long i)
{
	return FPR(1.0 / (double)i);
}

static const fpr fpr_log2 = { 0.69314718055994530941723212146 };
static const fpr fpr_p55 = { 36028797018963968.0 };
static const fpr fpr_p63 = { 9223372036854775808.0 };
static const fpr fpr_p64 = { 18446744073709551616.0 };

/*
 * For w = exp(i*pi/3), real and imaginary parts of w^1, w^2, w^4 and w^5.
 *
 *   w^2 and w^4 are the two primitive cubic roots of 1.
 *
 *   w^1 and w^5 are the two roots of X^2-X+1.
 */
static const fpr fpr_W1R = {  0.500000000000000000000000000 };
static const fpr fpr_W1I = {  0.866025403784438646763723171 };
static const fpr fpr_W2R = { -0.500000000000000000000000000 };
static const fpr fpr_W2I = {  0.866025403784438646763723171 };
static const fpr fpr_W4R = { -0.500000000000000000000000000 };
static const fpr fpr_W4I = { -0.866025403784438646763723171 };
static const fpr fpr_W5R = {  0.500000000000000000000000000 };
static const fpr fpr_W5I = { -0.866025403784438646763723171 };

/*
 * For w = exp(i*pi/3), the coefficient c = Re(w)/Im(w).
 */
static const fpr fpr_IW1I = {  1.154700538379251529018297561 };

static inline int64_t
fpr_rint(fpr x)
{
	return llrint(x.v);
}

static inline long
fpr_floor(fpr x)
{
	return (long)floor(x.v);
}

static inline fpr
fpr_add(fpr x, fpr y)
{
	return FPR(x.v + y.v);
}

static inline fpr
fpr_sub(fpr x, fpr y)
{
	return FPR(x.v - y.v);
}

static inline fpr
fpr_neg(fpr x)
{
	return FPR(-x.v);
}

static inline fpr
fpr_half(fpr x)
{
	return FPR(x.v * 0.5);
}

static inline fpr
fpr_double(fpr x)
{
	return FPR(x.v + x.v);
}

static inline fpr
fpr_mul(fpr x, fpr y)
{
	return FPR(x.v * y.v);
}

static inline fpr
fpr_sqr(fpr x)
{
	return FPR(x.v * x.v);
}

static inline fpr
fpr_inv(fpr x)
{
	return FPR(1.0 / x.v);
}

static inline fpr
fpr_div(fpr x, fpr y)
{
	return FPR(x.v / y.v);
}

static inline fpr
fpr_sqrt(fpr x)
{
	return FPR(sqrt(x.v));
}

static inline fpr
fpr_max(fpr x, fpr y)
{
	return FPR(fmax(x.v, y.v));
}

static inline int
fpr_lt(fpr x, fpr y)
{
	return x.v < y.v;
}

/*
 * Compute exp(x) for x such that |x| <= ln 2. We want a precision of 50
 * bits or so.
 */
static inline fpr
fpr_exp_small(fpr x)
{
	/*
	 * The algorithm used below is derived from the public domain
	 * library fdlibm (http://www.netlib.org/fdlibm/e_exp.c).
	 *
	 * We compute exp(x/2) to make sure that the value is in the
	 * proper range for the polynomial approximation, then square
	 * it to get exp(x).
	 *
	 * This is constant-time IF the base floating-point operations
	 * are constant-time (which is a big "if", especially for the
	 * division operation).
	 */
#define FPR_P1   ( 1.66666666666666019037e-01)
#define FPR_P2   (-2.77777777770155933842e-03)
#define FPR_P3   ( 6.61375632143793436117e-05)
#define FPR_P4   (-1.65339022054652515390e-06)
#define FPR_P5   ( 4.13813679705723846039e-08)

	double s, t, c;

	s = x.v;
	s *= 0.5;
	t = s * s;
	c = s - t * (FPR_P1 + t * (FPR_P2
		+ t * (FPR_P3 + t * (FPR_P4 + t * FPR_P5))));
	s = 1.0 - ((s * c) / (c - 2.0) - s);
	return FPR(s * s);

#undef FPR_P1
#undef FPR_P2
#undef FPR_P3
#undef FPR_P4
#undef FPR_P5
}

static inline void
fpr_gauss(fpr *re, fpr *im, fpr sigma, uint32_t a, uint32_t b)
{
	/*
	 * FIXME: import code from fdlibm to avoid dependency on libm
	 * functions log(), cos() and sin().
	 */
	double r, phi;

	r = ldexp((double)a + 1, -32);
	r = sigma.v * sqrt(-2.0 * log(r));
	phi = ldexp((double)b + 1, -32) * 6.283185307179586476925;
	*re = FPR(r * cos(phi));
	*im = FPR(r * sin(phi));
}

#else

static const fpr fpr_gm_tab[] = {
	FPC({0}, {0}), /* unused */
	FPC({-0.000000000000000000000000000}, { 1.000000000000000000000000000}),
	FPC({ 0.707106781186547524400844362}, { 0.707106781186547524400844362}),
	FPC({-0.707106781186547524400844362}, { 0.707106781186547524400844362}),
	FPC({ 0.923879532511286756128183189}, { 0.382683432365089771728459984}),
	FPC({-0.382683432365089771728459984}, { 0.923879532511286756128183189}),
	FPC({ 0.382683432365089771728459984}, { 0.923879532511286756128183189}),
	FPC({-0.923879532511286756128183189}, { 0.382683432365089771728459984}),
	FPC({ 0.980785280403230449126182236}, { 0.195090322016128267848284868}),
	FPC({-0.195090322016128267848284868}, { 0.980785280403230449126182236}),
	FPC({ 0.555570233019602224742830814}, { 0.831469612302545237078788378}),
	FPC({-0.831469612302545237078788378}, { 0.555570233019602224742830814}),
	FPC({ 0.831469612302545237078788378}, { 0.555570233019602224742830814}),
	FPC({-0.555570233019602224742830814}, { 0.831469612302545237078788378}),
	FPC({ 0.195090322016128267848284868}, { 0.980785280403230449126182236}),
	FPC({-0.980785280403230449126182236}, { 0.195090322016128267848284868}),
	FPC({ 0.995184726672196886244836953}, { 0.098017140329560601994195564}),
	FPC({-0.098017140329560601994195564}, { 0.995184726672196886244836953}),
	FPC({ 0.634393284163645498215171613}, { 0.773010453362736960810906610}),
	FPC({-0.773010453362736960810906610}, { 0.634393284163645498215171613}),
	FPC({ 0.881921264348355029712756864}, { 0.471396736825997648556387626}),
	FPC({-0.471396736825997648556387626}, { 0.881921264348355029712756864}),
	FPC({ 0.290284677254462367636192376}, { 0.956940335732208864935797887}),
	FPC({-0.956940335732208864935797887}, { 0.290284677254462367636192376}),
	FPC({ 0.956940335732208864935797887}, { 0.290284677254462367636192376}),
	FPC({-0.290284677254462367636192376}, { 0.956940335732208864935797887}),
	FPC({ 0.471396736825997648556387626}, { 0.881921264348355029712756864}),
	FPC({-0.881921264348355029712756864}, { 0.471396736825997648556387626}),
	FPC({ 0.773010453362736960810906610}, { 0.634393284163645498215171613}),
	FPC({-0.634393284163645498215171613}, { 0.773010453362736960810906610}),
	FPC({ 0.098017140329560601994195564}, { 0.995184726672196886244836953}),
	FPC({-0.995184726672196886244836953}, { 0.098017140329560601994195564}),
	FPC({ 0.998795456205172392714771605}, { 0.049067674327418014254954977}),
	FPC({-0.049067674327418014254954977}, { 0.998795456205172392714771605}),
	FPC({ 0.671558954847018400625376850}, { 0.740951125354959091175616897}),
	FPC({-0.740951125354959091175616897}, { 0.671558954847018400625376850}),
	FPC({ 0.903989293123443331586200297}, { 0.427555093430282094320966857}),
	FPC({-0.427555093430282094320966857}, { 0.903989293123443331586200297}),
	FPC({ 0.336889853392220050689253213}, { 0.941544065183020778412509403}),
	FPC({-0.941544065183020778412509403}, { 0.336889853392220050689253213}),
	FPC({ 0.970031253194543992603984207}, { 0.242980179903263889948274162}),
	FPC({-0.242980179903263889948274162}, { 0.970031253194543992603984207}),
	FPC({ 0.514102744193221726593693839}, { 0.857728610000272069902269984}),
	FPC({-0.857728610000272069902269984}, { 0.514102744193221726593693839}),
	FPC({ 0.803207531480644909806676513}, { 0.595699304492433343467036529}),
	FPC({-0.595699304492433343467036529}, { 0.803207531480644909806676513}),
	FPC({ 0.146730474455361751658850130}, { 0.989176509964780973451673738}),
	FPC({-0.989176509964780973451673738}, { 0.146730474455361751658850130}),
	FPC({ 0.989176509964780973451673738}, { 0.146730474455361751658850130}),
	FPC({-0.146730474455361751658850130}, { 0.989176509964780973451673738}),
	FPC({ 0.595699304492433343467036529}, { 0.803207531480644909806676513}),
	FPC({-0.803207531480644909806676513}, { 0.595699304492433343467036529}),
	FPC({ 0.857728610000272069902269984}, { 0.514102744193221726593693839}),
	FPC({-0.514102744193221726593693839}, { 0.857728610000272069902269984}),
	FPC({ 0.242980179903263889948274162}, { 0.970031253194543992603984207}),
	FPC({-0.970031253194543992603984207}, { 0.242980179903263889948274162}),
	FPC({ 0.941544065183020778412509403}, { 0.336889853392220050689253213}),
	FPC({-0.336889853392220050689253213}, { 0.941544065183020778412509403}),
	FPC({ 0.427555093430282094320966857}, { 0.903989293123443331586200297}),
	FPC({-0.903989293123443331586200297}, { 0.427555093430282094320966857}),
	FPC({ 0.740951125354959091175616897}, { 0.671558954847018400625376850}),
	FPC({-0.671558954847018400625376850}, { 0.740951125354959091175616897}),
	FPC({ 0.049067674327418014254954977}, { 0.998795456205172392714771605}),
	FPC({-0.998795456205172392714771605}, { 0.049067674327418014254954977}),
	FPC({ 0.999698818696204220115765650}, { 0.024541228522912288031734529}),
	FPC({-0.024541228522912288031734529}, { 0.999698818696204220115765650}),
	FPC({ 0.689540544737066924616730630}, { 0.724247082951466920941069243}),
	FPC({-0.724247082951466920941069243}, { 0.689540544737066924616730630}),
	FPC({ 0.914209755703530654635014829}, { 0.405241314004989870908481306}),
	FPC({-0.405241314004989870908481306}, { 0.914209755703530654635014829}),
	FPC({ 0.359895036534988148775104572}, { 0.932992798834738887711660256}),
	FPC({-0.932992798834738887711660256}, { 0.359895036534988148775104572}),
	FPC({ 0.975702130038528544460395766}, { 0.219101240156869797227737547}),
	FPC({-0.219101240156869797227737547}, { 0.975702130038528544460395766}),
	FPC({ 0.534997619887097210663076905}, { 0.844853565249707073259571205}),
	FPC({-0.844853565249707073259571205}, { 0.534997619887097210663076905}),
	FPC({ 0.817584813151583696504920884}, { 0.575808191417845300745972454}),
	FPC({-0.575808191417845300745972454}, { 0.817584813151583696504920884}),
	FPC({ 0.170961888760301226363642357}, { 0.985277642388941244774018433}),
	FPC({-0.985277642388941244774018433}, { 0.170961888760301226363642357}),
	FPC({ 0.992479534598709998156767252}, { 0.122410675199216198498704474}),
	FPC({-0.122410675199216198498704474}, { 0.992479534598709998156767252}),
	FPC({ 0.615231590580626845484913563}, { 0.788346427626606262009164705}),
	FPC({-0.788346427626606262009164705}, { 0.615231590580626845484913563}),
	FPC({ 0.870086991108711418652292404}, { 0.492898192229784036873026689}),
	FPC({-0.492898192229784036873026689}, { 0.870086991108711418652292404}),
	FPC({ 0.266712757474898386325286515}, { 0.963776065795439866686464356}),
	FPC({-0.963776065795439866686464356}, { 0.266712757474898386325286515}),
	FPC({ 0.949528180593036667195936074}, { 0.313681740398891476656478846}),
	FPC({-0.313681740398891476656478846}, { 0.949528180593036667195936074}),
	FPC({ 0.449611329654606600046294579}, { 0.893224301195515320342416447}),
	FPC({-0.893224301195515320342416447}, { 0.449611329654606600046294579}),
	FPC({ 0.757208846506484547575464054}, { 0.653172842953776764084203014}),
	FPC({-0.653172842953776764084203014}, { 0.757208846506484547575464054}),
	FPC({ 0.073564563599667423529465622}, { 0.997290456678690216135597140}),
	FPC({-0.997290456678690216135597140}, { 0.073564563599667423529465622}),
	FPC({ 0.997290456678690216135597140}, { 0.073564563599667423529465622}),
	FPC({-0.073564563599667423529465622}, { 0.997290456678690216135597140}),
	FPC({ 0.653172842953776764084203014}, { 0.757208846506484547575464054}),
	FPC({-0.757208846506484547575464054}, { 0.653172842953776764084203014}),
	FPC({ 0.893224301195515320342416447}, { 0.449611329654606600046294579}),
	FPC({-0.449611329654606600046294579}, { 0.893224301195515320342416447}),
	FPC({ 0.313681740398891476656478846}, { 0.949528180593036667195936074}),
	FPC({-0.949528180593036667195936074}, { 0.313681740398891476656478846}),
	FPC({ 0.963776065795439866686464356}, { 0.266712757474898386325286515}),
	FPC({-0.266712757474898386325286515}, { 0.963776065795439866686464356}),
	FPC({ 0.492898192229784036873026689}, { 0.870086991108711418652292404}),
	FPC({-0.870086991108711418652292404}, { 0.492898192229784036873026689}),
	FPC({ 0.788346427626606262009164705}, { 0.615231590580626845484913563}),
	FPC({-0.615231590580626845484913563}, { 0.788346427626606262009164705}),
	FPC({ 0.122410675199216198498704474}, { 0.992479534598709998156767252}),
	FPC({-0.992479534598709998156767252}, { 0.122410675199216198498704474}),
	FPC({ 0.985277642388941244774018433}, { 0.170961888760301226363642357}),
	FPC({-0.170961888760301226363642357}, { 0.985277642388941244774018433}),
	FPC({ 0.575808191417845300745972454}, { 0.817584813151583696504920884}),
	FPC({-0.817584813151583696504920884}, { 0.575808191417845300745972454}),
	FPC({ 0.844853565249707073259571205}, { 0.534997619887097210663076905}),
	FPC({-0.534997619887097210663076905}, { 0.844853565249707073259571205}),
	FPC({ 0.219101240156869797227737547}, { 0.975702130038528544460395766}),
	FPC({-0.975702130038528544460395766}, { 0.219101240156869797227737547}),
	FPC({ 0.932992798834738887711660256}, { 0.359895036534988148775104572}),
	FPC({-0.359895036534988148775104572}, { 0.932992798834738887711660256}),
	FPC({ 0.405241314004989870908481306}, { 0.914209755703530654635014829}),
	FPC({-0.914209755703530654635014829}, { 0.405241314004989870908481306}),
	FPC({ 0.724247082951466920941069243}, { 0.689540544737066924616730630}),
	FPC({-0.689540544737066924616730630}, { 0.724247082951466920941069243}),
	FPC({ 0.024541228522912288031734529}, { 0.999698818696204220115765650}),
	FPC({-0.999698818696204220115765650}, { 0.024541228522912288031734529}),
	FPC({ 0.999924701839144540921646491}, { 0.012271538285719926079408262}),
	FPC({-0.012271538285719926079408262}, { 0.999924701839144540921646491}),
	FPC({ 0.698376249408972853554813503}, { 0.715730825283818654125532623}),
	FPC({-0.715730825283818654125532623}, { 0.698376249408972853554813503}),
	FPC({ 0.919113851690057743908477789}, { 0.393992040061048108596188661}),
	FPC({-0.393992040061048108596188661}, { 0.919113851690057743908477789}),
	FPC({ 0.371317193951837543411934967}, { 0.928506080473215565937167396}),
	FPC({-0.928506080473215565937167396}, { 0.371317193951837543411934967}),
	FPC({ 0.978317370719627633106240097}, { 0.207111376192218549708116020}),
	FPC({-0.207111376192218549708116020}, { 0.978317370719627633106240097}),
	FPC({ 0.545324988422046422313987347}, { 0.838224705554838043186996856}),
	FPC({-0.838224705554838043186996856}, { 0.545324988422046422313987347}),
	FPC({ 0.824589302785025264474803737}, { 0.565731810783613197389765011}),
	FPC({-0.565731810783613197389765011}, { 0.824589302785025264474803737}),
	FPC({ 0.183039887955140958516532578}, { 0.983105487431216327180301155}),
	FPC({-0.983105487431216327180301155}, { 0.183039887955140958516532578}),
	FPC({ 0.993906970002356041546922813}, { 0.110222207293883058807899140}),
	FPC({-0.110222207293883058807899140}, { 0.993906970002356041546922813}),
	FPC({ 0.624859488142386377084072816}, { 0.780737228572094478301588484}),
	FPC({-0.780737228572094478301588484}, { 0.624859488142386377084072816}),
	FPC({ 0.876070094195406607095844268}, { 0.482183772079122748517344481}),
	FPC({-0.482183772079122748517344481}, { 0.876070094195406607095844268}),
	FPC({ 0.278519689385053105207848526}, { 0.960430519415565811199035138}),
	FPC({-0.960430519415565811199035138}, { 0.278519689385053105207848526}),
	FPC({ 0.953306040354193836916740383}, { 0.302005949319228067003463232}),
	FPC({-0.302005949319228067003463232}, { 0.953306040354193836916740383}),
	FPC({ 0.460538710958240023633181487}, { 0.887639620402853947760181617}),
	FPC({-0.887639620402853947760181617}, { 0.460538710958240023633181487}),
	FPC({ 0.765167265622458925888815999}, { 0.643831542889791465068086063}),
	FPC({-0.643831542889791465068086063}, { 0.765167265622458925888815999}),
	FPC({ 0.085797312344439890461556332}, { 0.996312612182778012627226190}),
	FPC({-0.996312612182778012627226190}, { 0.085797312344439890461556332}),
	FPC({ 0.998118112900149207125155861}, { 0.061320736302208577782614593}),
	FPC({-0.061320736302208577782614593}, { 0.998118112900149207125155861}),
	FPC({ 0.662415777590171761113069817}, { 0.749136394523459325469203257}),
	FPC({-0.749136394523459325469203257}, { 0.662415777590171761113069817}),
	FPC({ 0.898674465693953843041976744}, { 0.438616238538527637647025738}),
	FPC({-0.438616238538527637647025738}, { 0.898674465693953843041976744}),
	FPC({ 0.325310292162262934135954708}, { 0.945607325380521325730945387}),
	FPC({-0.945607325380521325730945387}, { 0.325310292162262934135954708}),
	FPC({ 0.966976471044852109087220226}, { 0.254865659604514571553980779}),
	FPC({-0.254865659604514571553980779}, { 0.966976471044852109087220226}),
	FPC({ 0.503538383725717558691867071}, { 0.863972856121586737918147054}),
	FPC({-0.863972856121586737918147054}, { 0.503538383725717558691867071}),
	FPC({ 0.795836904608883536262791915}, { 0.605511041404325513920626941}),
	FPC({-0.605511041404325513920626941}, { 0.795836904608883536262791915}),
	FPC({ 0.134580708507126186316358409}, { 0.990902635427780025108237011}),
	FPC({-0.990902635427780025108237011}, { 0.134580708507126186316358409}),
	FPC({ 0.987301418157858382399815802}, { 0.158858143333861441684385360}),
	FPC({-0.158858143333861441684385360}, { 0.987301418157858382399815802}),
	FPC({ 0.585797857456438860328080838}, { 0.810457198252594791726703434}),
	FPC({-0.810457198252594791726703434}, { 0.585797857456438860328080838}),
	FPC({ 0.851355193105265142261290312}, { 0.524589682678468906215098464}),
	FPC({-0.524589682678468906215098464}, { 0.851355193105265142261290312}),
	FPC({ 0.231058108280671119643236018}, { 0.972939952205560145467720114}),
	FPC({-0.972939952205560145467720114}, { 0.231058108280671119643236018}),
	FPC({ 0.937339011912574923201899593}, { 0.348418680249434568419308588}),
	FPC({-0.348418680249434568419308588}, { 0.937339011912574923201899593}),
	FPC({ 0.416429560097637182562598911}, { 0.909167983090522376563884788}),
	FPC({-0.909167983090522376563884788}, { 0.416429560097637182562598911}),
	FPC({ 0.732654271672412834615546649}, { 0.680600997795453050594430464}),
	FPC({-0.680600997795453050594430464}, { 0.732654271672412834615546649}),
	FPC({ 0.036807222941358832324332691}, { 0.999322384588349500896221011}),
	FPC({-0.999322384588349500896221011}, { 0.036807222941358832324332691}),
	FPC({ 0.999322384588349500896221011}, { 0.036807222941358832324332691}),
	FPC({-0.036807222941358832324332691}, { 0.999322384588349500896221011}),
	FPC({ 0.680600997795453050594430464}, { 0.732654271672412834615546649}),
	FPC({-0.732654271672412834615546649}, { 0.680600997795453050594430464}),
	FPC({ 0.909167983090522376563884788}, { 0.416429560097637182562598911}),
	FPC({-0.416429560097637182562598911}, { 0.909167983090522376563884788}),
	FPC({ 0.348418680249434568419308588}, { 0.937339011912574923201899593}),
	FPC({-0.937339011912574923201899593}, { 0.348418680249434568419308588}),
	FPC({ 0.972939952205560145467720114}, { 0.231058108280671119643236018}),
	FPC({-0.231058108280671119643236018}, { 0.972939952205560145467720114}),
	FPC({ 0.524589682678468906215098464}, { 0.851355193105265142261290312}),
	FPC({-0.851355193105265142261290312}, { 0.524589682678468906215098464}),
	FPC({ 0.810457198252594791726703434}, { 0.585797857456438860328080838}),
	FPC({-0.585797857456438860328080838}, { 0.810457198252594791726703434}),
	FPC({ 0.158858143333861441684385360}, { 0.987301418157858382399815802}),
	FPC({-0.987301418157858382399815802}, { 0.158858143333861441684385360}),
	FPC({ 0.990902635427780025108237011}, { 0.134580708507126186316358409}),
	FPC({-0.134580708507126186316358409}, { 0.990902635427780025108237011}),
	FPC({ 0.605511041404325513920626941}, { 0.795836904608883536262791915}),
	FPC({-0.795836904608883536262791915}, { 0.605511041404325513920626941}),
	FPC({ 0.863972856121586737918147054}, { 0.503538383725717558691867071}),
	FPC({-0.503538383725717558691867071}, { 0.863972856121586737918147054}),
	FPC({ 0.254865659604514571553980779}, { 0.966976471044852109087220226}),
	FPC({-0.966976471044852109087220226}, { 0.254865659604514571553980779}),
	FPC({ 0.945607325380521325730945387}, { 0.325310292162262934135954708}),
	FPC({-0.325310292162262934135954708}, { 0.945607325380521325730945387}),
	FPC({ 0.438616238538527637647025738}, { 0.898674465693953843041976744}),
	FPC({-0.898674465693953843041976744}, { 0.438616238538527637647025738}),
	FPC({ 0.749136394523459325469203257}, { 0.662415777590171761113069817}),
	FPC({-0.662415777590171761113069817}, { 0.749136394523459325469203257}),
	FPC({ 0.061320736302208577782614593}, { 0.998118112900149207125155861}),
	FPC({-0.998118112900149207125155861}, { 0.061320736302208577782614593}),
	FPC({ 0.996312612182778012627226190}, { 0.085797312344439890461556332}),
	FPC({-0.085797312344439890461556332}, { 0.996312612182778012627226190}),
	FPC({ 0.643831542889791465068086063}, { 0.765167265622458925888815999}),
	FPC({-0.765167265622458925888815999}, { 0.643831542889791465068086063}),
	FPC({ 0.887639620402853947760181617}, { 0.460538710958240023633181487}),
	FPC({-0.460538710958240023633181487}, { 0.887639620402853947760181617}),
	FPC({ 0.302005949319228067003463232}, { 0.953306040354193836916740383}),
	FPC({-0.953306040354193836916740383}, { 0.302005949319228067003463232}),
	FPC({ 0.960430519415565811199035138}, { 0.278519689385053105207848526}),
	FPC({-0.278519689385053105207848526}, { 0.960430519415565811199035138}),
	FPC({ 0.482183772079122748517344481}, { 0.876070094195406607095844268}),
	FPC({-0.876070094195406607095844268}, { 0.482183772079122748517344481}),
	FPC({ 0.780737228572094478301588484}, { 0.624859488142386377084072816}),
	FPC({-0.624859488142386377084072816}, { 0.780737228572094478301588484}),
	FPC({ 0.110222207293883058807899140}, { 0.993906970002356041546922813}),
	FPC({-0.993906970002356041546922813}, { 0.110222207293883058807899140}),
	FPC({ 0.983105487431216327180301155}, { 0.183039887955140958516532578}),
	FPC({-0.183039887955140958516532578}, { 0.983105487431216327180301155}),
	FPC({ 0.565731810783613197389765011}, { 0.824589302785025264474803737}),
	FPC({-0.824589302785025264474803737}, { 0.565731810783613197389765011}),
	FPC({ 0.838224705554838043186996856}, { 0.545324988422046422313987347}),
	FPC({-0.545324988422046422313987347}, { 0.838224705554838043186996856}),
	FPC({ 0.207111376192218549708116020}, { 0.978317370719627633106240097}),
	FPC({-0.978317370719627633106240097}, { 0.207111376192218549708116020}),
	FPC({ 0.928506080473215565937167396}, { 0.371317193951837543411934967}),
	FPC({-0.371317193951837543411934967}, { 0.928506080473215565937167396}),
	FPC({ 0.393992040061048108596188661}, { 0.919113851690057743908477789}),
	FPC({-0.919113851690057743908477789}, { 0.393992040061048108596188661}),
	FPC({ 0.715730825283818654125532623}, { 0.698376249408972853554813503}),
	FPC({-0.698376249408972853554813503}, { 0.715730825283818654125532623}),
	FPC({ 0.012271538285719926079408262}, { 0.999924701839144540921646491}),
	FPC({-0.999924701839144540921646491}, { 0.012271538285719926079408262}),
	FPC({ 0.999981175282601142656990438}, { 0.006135884649154475359640235}),
	FPC({-0.006135884649154475359640235}, { 0.999981175282601142656990438}),
	FPC({ 0.702754744457225302452914421}, { 0.711432195745216441522130290}),
	FPC({-0.711432195745216441522130290}, { 0.702754744457225302452914421}),
	FPC({ 0.921514039342041943465396332}, { 0.388345046698826291624993541}),
	FPC({-0.388345046698826291624993541}, { 0.921514039342041943465396332}),
	FPC({ 0.377007410216418256726567823}, { 0.926210242138311341974793388}),
	FPC({-0.926210242138311341974793388}, { 0.377007410216418256726567823}),
	FPC({ 0.979569765685440534439326110}, { 0.201104634842091911558443546}),
	FPC({-0.201104634842091911558443546}, { 0.979569765685440534439326110}),
	FPC({ 0.550457972936604802977289893}, { 0.834862874986380056304401383}),
	FPC({-0.834862874986380056304401383}, { 0.550457972936604802977289893}),
	FPC({ 0.828045045257755752067527592}, { 0.560661576197336023839710223}),
	FPC({-0.560661576197336023839710223}, { 0.828045045257755752067527592}),
	FPC({ 0.189068664149806212754997837}, { 0.981963869109555264072848154}),
	FPC({-0.981963869109555264072848154}, { 0.189068664149806212754997837}),
	FPC({ 0.994564570734255452119106243}, { 0.104121633872054579120943880}),
	FPC({-0.104121633872054579120943880}, { 0.994564570734255452119106243}),
	FPC({ 0.629638238914927025372981341}, { 0.776888465673232450040827983}),
	FPC({-0.776888465673232450040827983}, { 0.629638238914927025372981341}),
	FPC({ 0.879012226428633477831323711}, { 0.476799230063322133342158117}),
	FPC({-0.476799230063322133342158117}, { 0.879012226428633477831323711}),
	FPC({ 0.284407537211271843618310615}, { 0.958703474895871555374645792}),
	FPC({-0.958703474895871555374645792}, { 0.284407537211271843618310615}),
	FPC({ 0.955141168305770721498157712}, { 0.296150888243623824121786128}),
	FPC({-0.296150888243623824121786128}, { 0.955141168305770721498157712}),
	FPC({ 0.465976495767966177902756065}, { 0.884797098430937780104007041}),
	FPC({-0.884797098430937780104007041}, { 0.465976495767966177902756065}),
	FPC({ 0.769103337645579639346626069}, { 0.639124444863775743801488193}),
	FPC({-0.639124444863775743801488193}, { 0.769103337645579639346626069}),
	FPC({ 0.091908956497132728624990979}, { 0.995767414467659793982495643}),
	FPC({-0.995767414467659793982495643}, { 0.091908956497132728624990979}),
	FPC({ 0.998475580573294752208559038}, { 0.055195244349689939809447526}),
	FPC({-0.055195244349689939809447526}, { 0.998475580573294752208559038}),
	FPC({ 0.666999922303637506650154222}, { 0.745057785441465962407907310}),
	FPC({-0.745057785441465962407907310}, { 0.666999922303637506650154222}),
	FPC({ 0.901348847046022014570746093}, { 0.433093818853151968484222638}),
	FPC({-0.433093818853151968484222638}, { 0.901348847046022014570746093}),
	FPC({ 0.331106305759876401737190737}, { 0.943593458161960361495301445}),
	FPC({-0.943593458161960361495301445}, { 0.331106305759876401737190737}),
	FPC({ 0.968522094274417316221088329}, { 0.248927605745720168110682816}),
	FPC({-0.248927605745720168110682816}, { 0.968522094274417316221088329}),
	FPC({ 0.508830142543107036931749324}, { 0.860866938637767279344583877}),
	FPC({-0.860866938637767279344583877}, { 0.508830142543107036931749324}),
	FPC({ 0.799537269107905033500246232}, { 0.600616479383868926653875896}),
	FPC({-0.600616479383868926653875896}, { 0.799537269107905033500246232}),
	FPC({ 0.140658239332849230714788846}, { 0.990058210262297105505906464}),
	FPC({-0.990058210262297105505906464}, { 0.140658239332849230714788846}),
	FPC({ 0.988257567730749491404792538}, { 0.152797185258443427720336613}),
	FPC({-0.152797185258443427720336613}, { 0.988257567730749491404792538}),
	FPC({ 0.590759701858874228423887908}, { 0.806847553543799272206514313}),
	FPC({-0.806847553543799272206514313}, { 0.590759701858874228423887908}),
	FPC({ 0.854557988365400520767862276}, { 0.519355990165589587361829932}),
	FPC({-0.519355990165589587361829932}, { 0.854557988365400520767862276}),
	FPC({ 0.237023605994367206867735915}, { 0.971503890986251775537099622}),
	FPC({-0.971503890986251775537099622}, { 0.237023605994367206867735915}),
	FPC({ 0.939459223602189911962669246}, { 0.342660717311994397592781983}),
	FPC({-0.342660717311994397592781983}, { 0.939459223602189911962669246}),
	FPC({ 0.422000270799799685941287941}, { 0.906595704514915365332960588}),
	FPC({-0.906595704514915365332960588}, { 0.422000270799799685941287941}),
	FPC({ 0.736816568877369875090132520}, { 0.676092703575315960360419228}),
	FPC({-0.676092703575315960360419228}, { 0.736816568877369875090132520}),
	FPC({ 0.042938256934940823077124540}, { 0.999077727752645382888781997}),
	FPC({-0.999077727752645382888781997}, { 0.042938256934940823077124540}),
	FPC({ 0.999529417501093163079703322}, { 0.030674803176636625934021028}),
	FPC({-0.030674803176636625934021028}, { 0.999529417501093163079703322}),
	FPC({ 0.685083667772700381362052545}, { 0.728464390448225196492035438}),
	FPC({-0.728464390448225196492035438}, { 0.685083667772700381362052545}),
	FPC({ 0.911706032005429851404397325}, { 0.410843171057903942183466675}),
	FPC({-0.410843171057903942183466675}, { 0.911706032005429851404397325}),
	FPC({ 0.354163525420490382357395796}, { 0.935183509938947577642207480}),
	FPC({-0.935183509938947577642207480}, { 0.354163525420490382357395796}),
	FPC({ 0.974339382785575860518721668}, { 0.225083911359792835991642120}),
	FPC({-0.225083911359792835991642120}, { 0.974339382785575860518721668}),
	FPC({ 0.529803624686294668216054671}, { 0.848120344803297251279133563}),
	FPC({-0.848120344803297251279133563}, { 0.529803624686294668216054671}),
	FPC({ 0.814036329705948361654516690}, { 0.580813958095764545075595272}),
	FPC({-0.580813958095764545075595272}, { 0.814036329705948361654516690}),
	FPC({ 0.164913120489969921418189113}, { 0.986308097244598647863297524}),
	FPC({-0.986308097244598647863297524}, { 0.164913120489969921418189113}),
	FPC({ 0.991709753669099522860049931}, { 0.128498110793793172624415589}),
	FPC({-0.128498110793793172624415589}, { 0.991709753669099522860049931}),
	FPC({ 0.610382806276309452716352152}, { 0.792106577300212351782342879}),
	FPC({-0.792106577300212351782342879}, { 0.610382806276309452716352152}),
	FPC({ 0.867046245515692651480195629}, { 0.498227666972781852410983869}),
	FPC({-0.498227666972781852410983869}, { 0.867046245515692651480195629}),
	FPC({ 0.260794117915275518280186509}, { 0.965394441697689374550843858}),
	FPC({-0.965394441697689374550843858}, { 0.260794117915275518280186509}),
	FPC({ 0.947585591017741134653387321}, { 0.319502030816015677901518272}),
	FPC({-0.319502030816015677901518272}, { 0.947585591017741134653387321}),
	FPC({ 0.444122144570429231642069418}, { 0.895966249756185155914560282}),
	FPC({-0.895966249756185155914560282}, { 0.444122144570429231642069418}),
	FPC({ 0.753186799043612482483430486}, { 0.657806693297078656931182264}),
	FPC({-0.657806693297078656931182264}, { 0.753186799043612482483430486}),
	FPC({ 0.067443919563664057897972422}, { 0.997723066644191609848546728}),
	FPC({-0.997723066644191609848546728}, { 0.067443919563664057897972422}),
	FPC({ 0.996820299291165714972629398}, { 0.079682437971430121147120656}),
	FPC({-0.079682437971430121147120656}, { 0.996820299291165714972629398}),
	FPC({ 0.648514401022112445084560551}, { 0.761202385484261814029709836}),
	FPC({-0.761202385484261814029709836}, { 0.648514401022112445084560551}),
	FPC({ 0.890448723244757889952150560}, { 0.455083587126343823535869268}),
	FPC({-0.455083587126343823535869268}, { 0.890448723244757889952150560}),
	FPC({ 0.307849640041534893682063646}, { 0.951435020969008369549175569}),
	FPC({-0.951435020969008369549175569}, { 0.307849640041534893682063646}),
	FPC({ 0.962121404269041595429604316}, { 0.272621355449948984493347477}),
	FPC({-0.272621355449948984493347477}, { 0.962121404269041595429604316}),
	FPC({ 0.487550160148435954641485027}, { 0.873094978418290098636085973}),
	FPC({-0.873094978418290098636085973}, { 0.487550160148435954641485027}),
	FPC({ 0.784556597155575233023892575}, { 0.620057211763289178646268191}),
	FPC({-0.620057211763289178646268191}, { 0.784556597155575233023892575}),
	FPC({ 0.116318630911904767252544319}, { 0.993211949234794533104601012}),
	FPC({-0.993211949234794533104601012}, { 0.116318630911904767252544319}),
	FPC({ 0.984210092386929073193874387}, { 0.177004220412148756196839844}),
	FPC({-0.177004220412148756196839844}, { 0.984210092386929073193874387}),
	FPC({ 0.570780745886967280232652864}, { 0.821102514991104679060430820}),
	FPC({-0.821102514991104679060430820}, { 0.570780745886967280232652864}),
	FPC({ 0.841554977436898409603499520}, { 0.540171472729892881297845480}),
	FPC({-0.540171472729892881297845480}, { 0.841554977436898409603499520}),
	FPC({ 0.213110319916091373967757518}, { 0.977028142657754351485866211}),
	FPC({-0.977028142657754351485866211}, { 0.213110319916091373967757518}),
	FPC({ 0.930766961078983731944872340}, { 0.365612997804773870011745909}),
	FPC({-0.365612997804773870011745909}, { 0.930766961078983731944872340}),
	FPC({ 0.399624199845646828544117031}, { 0.916679059921042663116457013}),
	FPC({-0.916679059921042663116457013}, { 0.399624199845646828544117031}),
	FPC({ 0.720002507961381629076682999}, { 0.693971460889654009003734389}),
	FPC({-0.693971460889654009003734389}, { 0.720002507961381629076682999}),
	FPC({ 0.018406729905804820927366313}, { 0.999830581795823422015722275}),
	FPC({-0.999830581795823422015722275}, { 0.018406729905804820927366313}),
	FPC({ 0.999830581795823422015722275}, { 0.018406729905804820927366313}),
	FPC({-0.018406729905804820927366313}, { 0.999830581795823422015722275}),
	FPC({ 0.693971460889654009003734389}, { 0.720002507961381629076682999}),
	FPC({-0.720002507961381629076682999}, { 0.693971460889654009003734389}),
	FPC({ 0.916679059921042663116457013}, { 0.399624199845646828544117031}),
	FPC({-0.399624199845646828544117031}, { 0.916679059921042663116457013}),
	FPC({ 0.365612997804773870011745909}, { 0.930766961078983731944872340}),
	FPC({-0.930766961078983731944872340}, { 0.365612997804773870011745909}),
	FPC({ 0.977028142657754351485866211}, { 0.213110319916091373967757518}),
	FPC({-0.213110319916091373967757518}, { 0.977028142657754351485866211}),
	FPC({ 0.540171472729892881297845480}, { 0.841554977436898409603499520}),
	FPC({-0.841554977436898409603499520}, { 0.540171472729892881297845480}),
	FPC({ 0.821102514991104679060430820}, { 0.570780745886967280232652864}),
	FPC({-0.570780745886967280232652864}, { 0.821102514991104679060430820}),
	FPC({ 0.177004220412148756196839844}, { 0.984210092386929073193874387}),
	FPC({-0.984210092386929073193874387}, { 0.177004220412148756196839844}),
	FPC({ 0.993211949234794533104601012}, { 0.116318630911904767252544319}),
	FPC({-0.116318630911904767252544319}, { 0.993211949234794533104601012}),
	FPC({ 0.620057211763289178646268191}, { 0.784556597155575233023892575}),
	FPC({-0.784556597155575233023892575}, { 0.620057211763289178646268191}),
	FPC({ 0.873094978418290098636085973}, { 0.487550160148435954641485027}),
	FPC({-0.487550160148435954641485027}, { 0.873094978418290098636085973}),
	FPC({ 0.272621355449948984493347477}, { 0.962121404269041595429604316}),
	FPC({-0.962121404269041595429604316}, { 0.272621355449948984493347477}),
	FPC({ 0.951435020969008369549175569}, { 0.307849640041534893682063646}),
	FPC({-0.307849640041534893682063646}, { 0.951435020969008369549175569}),
	FPC({ 0.455083587126343823535869268}, { 0.890448723244757889952150560}),
	FPC({-0.890448723244757889952150560}, { 0.455083587126343823535869268}),
	FPC({ 0.761202385484261814029709836}, { 0.648514401022112445084560551}),
	FPC({-0.648514401022112445084560551}, { 0.761202385484261814029709836}),
	FPC({ 0.079682437971430121147120656}, { 0.996820299291165714972629398}),
	FPC({-0.996820299291165714972629398}, { 0.079682437971430121147120656}),
	FPC({ 0.997723066644191609848546728}, { 0.067443919563664057897972422}),
	FPC({-0.067443919563664057897972422}, { 0.997723066644191609848546728}),
	FPC({ 0.657806693297078656931182264}, { 0.753186799043612482483430486}),
	FPC({-0.753186799043612482483430486}, { 0.657806693297078656931182264}),
	FPC({ 0.895966249756185155914560282}, { 0.444122144570429231642069418}),
	FPC({-0.444122144570429231642069418}, { 0.895966249756185155914560282}),
	FPC({ 0.319502030816015677901518272}, { 0.947585591017741134653387321}),
	FPC({-0.947585591017741134653387321}, { 0.319502030816015677901518272}),
	FPC({ 0.965394441697689374550843858}, { 0.260794117915275518280186509}),
	FPC({-0.260794117915275518280186509}, { 0.965394441697689374550843858}),
	FPC({ 0.498227666972781852410983869}, { 0.867046245515692651480195629}),
	FPC({-0.867046245515692651480195629}, { 0.498227666972781852410983869}),
	FPC({ 0.792106577300212351782342879}, { 0.610382806276309452716352152}),
	FPC({-0.610382806276309452716352152}, { 0.792106577300212351782342879}),
	FPC({ 0.128498110793793172624415589}, { 0.991709753669099522860049931}),
	FPC({-0.991709753669099522860049931}, { 0.128498110793793172624415589}),
	FPC({ 0.986308097244598647863297524}, { 0.164913120489969921418189113}),
	FPC({-0.164913120489969921418189113}, { 0.986308097244598647863297524}),
	FPC({ 0.580813958095764545075595272}, { 0.814036329705948361654516690}),
	FPC({-0.814036329705948361654516690}, { 0.580813958095764545075595272}),
	FPC({ 0.848120344803297251279133563}, { 0.529803624686294668216054671}),
	FPC({-0.529803624686294668216054671}, { 0.848120344803297251279133563}),
	FPC({ 0.225083911359792835991642120}, { 0.974339382785575860518721668}),
	FPC({-0.974339382785575860518721668}, { 0.225083911359792835991642120}),
	FPC({ 0.935183509938947577642207480}, { 0.354163525420490382357395796}),
	FPC({-0.354163525420490382357395796}, { 0.935183509938947577642207480}),
	FPC({ 0.410843171057903942183466675}, { 0.911706032005429851404397325}),
	FPC({-0.911706032005429851404397325}, { 0.410843171057903942183466675}),
	FPC({ 0.728464390448225196492035438}, { 0.685083667772700381362052545}),
	FPC({-0.685083667772700381362052545}, { 0.728464390448225196492035438}),
	FPC({ 0.030674803176636625934021028}, { 0.999529417501093163079703322}),
	FPC({-0.999529417501093163079703322}, { 0.030674803176636625934021028}),
	FPC({ 0.999077727752645382888781997}, { 0.042938256934940823077124540}),
	FPC({-0.042938256934940823077124540}, { 0.999077727752645382888781997}),
	FPC({ 0.676092703575315960360419228}, { 0.736816568877369875090132520}),
	FPC({-0.736816568877369875090132520}, { 0.676092703575315960360419228}),
	FPC({ 0.906595704514915365332960588}, { 0.422000270799799685941287941}),
	FPC({-0.422000270799799685941287941}, { 0.906595704514915365332960588}),
	FPC({ 0.342660717311994397592781983}, { 0.939459223602189911962669246}),
	FPC({-0.939459223602189911962669246}, { 0.342660717311994397592781983}),
	FPC({ 0.971503890986251775537099622}, { 0.237023605994367206867735915}),
	FPC({-0.237023605994367206867735915}, { 0.971503890986251775537099622}),
	FPC({ 0.519355990165589587361829932}, { 0.854557988365400520767862276}),
	FPC({-0.854557988365400520767862276}, { 0.519355990165589587361829932}),
	FPC({ 0.806847553543799272206514313}, { 0.590759701858874228423887908}),
	FPC({-0.590759701858874228423887908}, { 0.806847553543799272206514313}),
	FPC({ 0.152797185258443427720336613}, { 0.988257567730749491404792538}),
	FPC({-0.988257567730749491404792538}, { 0.152797185258443427720336613}),
	FPC({ 0.990058210262297105505906464}, { 0.140658239332849230714788846}),
	FPC({-0.140658239332849230714788846}, { 0.990058210262297105505906464}),
	FPC({ 0.600616479383868926653875896}, { 0.799537269107905033500246232}),
	FPC({-0.799537269107905033500246232}, { 0.600616479383868926653875896}),
	FPC({ 0.860866938637767279344583877}, { 0.508830142543107036931749324}),
	FPC({-0.508830142543107036931749324}, { 0.860866938637767279344583877}),
	FPC({ 0.248927605745720168110682816}, { 0.968522094274417316221088329}),
	FPC({-0.968522094274417316221088329}, { 0.248927605745720168110682816}),
	FPC({ 0.943593458161960361495301445}, { 0.331106305759876401737190737}),
	FPC({-0.331106305759876401737190737}, { 0.943593458161960361495301445}),
	FPC({ 0.433093818853151968484222638}, { 0.901348847046022014570746093}),
	FPC({-0.901348847046022014570746093}, { 0.433093818853151968484222638}),
	FPC({ 0.745057785441465962407907310}, { 0.666999922303637506650154222}),
	FPC({-0.666999922303637506650154222}, { 0.745057785441465962407907310}),
	FPC({ 0.055195244349689939809447526}, { 0.998475580573294752208559038}),
	FPC({-0.998475580573294752208559038}, { 0.055195244349689939809447526}),
	FPC({ 0.995767414467659793982495643}, { 0.091908956497132728624990979}),
	FPC({-0.091908956497132728624990979}, { 0.995767414467659793982495643}),
	FPC({ 0.639124444863775743801488193}, { 0.769103337645579639346626069}),
	FPC({-0.769103337645579639346626069}, { 0.639124444863775743801488193}),
	FPC({ 0.884797098430937780104007041}, { 0.465976495767966177902756065}),
	FPC({-0.465976495767966177902756065}, { 0.884797098430937780104007041}),
	FPC({ 0.296150888243623824121786128}, { 0.955141168305770721498157712}),
	FPC({-0.955141168305770721498157712}, { 0.296150888243623824121786128}),
	FPC({ 0.958703474895871555374645792}, { 0.284407537211271843618310615}),
	FPC({-0.284407537211271843618310615}, { 0.958703474895871555374645792}),
	FPC({ 0.476799230063322133342158117}, { 0.879012226428633477831323711}),
	FPC({-0.879012226428633477831323711}, { 0.476799230063322133342158117}),
	FPC({ 0.776888465673232450040827983}, { 0.629638238914927025372981341}),
	FPC({-0.629638238914927025372981341}, { 0.776888465673232450040827983}),
	FPC({ 0.104121633872054579120943880}, { 0.994564570734255452119106243}),
	FPC({-0.994564570734255452119106243}, { 0.104121633872054579120943880}),
	FPC({ 0.981963869109555264072848154}, { 0.189068664149806212754997837}),
	FPC({-0.189068664149806212754997837}, { 0.981963869109555264072848154}),
	FPC({ 0.560661576197336023839710223}, { 0.828045045257755752067527592}),
	FPC({-0.828045045257755752067527592}, { 0.560661576197336023839710223}),
	FPC({ 0.834862874986380056304401383}, { 0.550457972936604802977289893}),
	FPC({-0.550457972936604802977289893}, { 0.834862874986380056304401383}),
	FPC({ 0.201104634842091911558443546}, { 0.979569765685440534439326110}),
	FPC({-0.979569765685440534439326110}, { 0.201104634842091911558443546}),
	FPC({ 0.926210242138311341974793388}, { 0.377007410216418256726567823}),
	FPC({-0.377007410216418256726567823}, { 0.926210242138311341974793388}),
	FPC({ 0.388345046698826291624993541}, { 0.921514039342041943465396332}),
	FPC({-0.921514039342041943465396332}, { 0.388345046698826291624993541}),
	FPC({ 0.711432195745216441522130290}, { 0.702754744457225302452914421}),
	FPC({-0.702754744457225302452914421}, { 0.711432195745216441522130290}),
	FPC({ 0.006135884649154475359640235}, { 0.999981175282601142656990438}),
	FPC({-0.999981175282601142656990438}, { 0.006135884649154475359640235}),
	FPC({ 0.999995293809576171511580126}, { 0.003067956762965976270145365}),
	FPC({-0.003067956762965976270145365}, { 0.999995293809576171511580126}),
	FPC({ 0.704934080375904908852523758}, { 0.709272826438865651316533772}),
	FPC({-0.709272826438865651316533772}, { 0.704934080375904908852523758}),
	FPC({ 0.922701128333878570437264227}, { 0.385516053843918864075607949}),
	FPC({-0.385516053843918864075607949}, { 0.922701128333878570437264227}),
	FPC({ 0.379847208924051170576281147}, { 0.925049240782677590302371869}),
	FPC({-0.925049240782677590302371869}, { 0.379847208924051170576281147}),
	FPC({ 0.980182135968117392690210009}, { 0.198098410717953586179324918}),
	FPC({-0.198098410717953586179324918}, { 0.980182135968117392690210009}),
	FPC({ 0.553016705580027531764226988}, { 0.833170164701913186439915922}),
	FPC({-0.833170164701913186439915922}, { 0.553016705580027531764226988}),
	FPC({ 0.829761233794523042469023765}, { 0.558118531220556115693702964}),
	FPC({-0.558118531220556115693702964}, { 0.829761233794523042469023765}),
	FPC({ 0.192080397049892441679288205}, { 0.981379193313754574318224190}),
	FPC({-0.981379193313754574318224190}, { 0.192080397049892441679288205}),
	FPC({ 0.994879330794805620591166107}, { 0.101069862754827824987887585}),
	FPC({-0.101069862754827824987887585}, { 0.994879330794805620591166107}),
	FPC({ 0.632018735939809021909403706}, { 0.774953106594873878359129282}),
	FPC({-0.774953106594873878359129282}, { 0.632018735939809021909403706}),
	FPC({ 0.880470889052160770806542929}, { 0.474100214650550014398580015}),
	FPC({-0.474100214650550014398580015}, { 0.880470889052160770806542929}),
	FPC({ 0.287347459544729526477331841}, { 0.957826413027532890321037029}),
	FPC({-0.957826413027532890321037029}, { 0.287347459544729526477331841}),
	FPC({ 0.956045251349996443270479823}, { 0.293219162694258650606608599}),
	FPC({-0.293219162694258650606608599}, { 0.956045251349996443270479823}),
	FPC({ 0.468688822035827933697617870}, { 0.883363338665731594736308015}),
	FPC({-0.883363338665731594736308015}, { 0.468688822035827933697617870}),
	FPC({ 0.771060524261813773200605759}, { 0.636761861236284230413943435}),
	FPC({-0.636761861236284230413943435}, { 0.771060524261813773200605759}),
	FPC({ 0.094963495329638998938034312}, { 0.995480755491926941769171600}),
	FPC({-0.995480755491926941769171600}, { 0.094963495329638998938034312}),
	FPC({ 0.998640218180265222418199049}, { 0.052131704680283321236358216}),
	FPC({-0.052131704680283321236358216}, { 0.998640218180265222418199049}),
	FPC({ 0.669282588346636065720696366}, { 0.743007952135121693517362293}),
	FPC({-0.743007952135121693517362293}, { 0.669282588346636065720696366}),
	FPC({ 0.902673318237258806751502391}, { 0.430326481340082633908199031}),
	FPC({-0.430326481340082633908199031}, { 0.902673318237258806751502391}),
	FPC({ 0.333999651442009404650865481}, { 0.942573197601446879280758735}),
	FPC({-0.942573197601446879280758735}, { 0.333999651442009404650865481}),
	FPC({ 0.969281235356548486048290738}, { 0.245955050335794611599924709}),
	FPC({-0.245955050335794611599924709}, { 0.969281235356548486048290738}),
	FPC({ 0.511468850437970399504391001}, { 0.859301818357008404783582139}),
	FPC({-0.859301818357008404783582139}, { 0.511468850437970399504391001}),
	FPC({ 0.801376171723140219430247777}, { 0.598160706996342311724958652}),
	FPC({-0.598160706996342311724958652}, { 0.801376171723140219430247777}),
	FPC({ 0.143695033150294454819773349}, { 0.989622017463200834623694454}),
	FPC({-0.989622017463200834623694454}, { 0.143695033150294454819773349}),
	FPC({ 0.988721691960323767604516485}, { 0.149764534677321517229695737}),
	FPC({-0.149764534677321517229695737}, { 0.988721691960323767604516485}),
	FPC({ 0.593232295039799808047809426}, { 0.805031331142963597922659282}),
	FPC({-0.805031331142963597922659282}, { 0.593232295039799808047809426}),
	FPC({ 0.856147328375194481019630732}, { 0.516731799017649881508753876}),
	FPC({-0.516731799017649881508753876}, { 0.856147328375194481019630732}),
	FPC({ 0.240003022448741486568922365}, { 0.970772140728950302138169611}),
	FPC({-0.970772140728950302138169611}, { 0.240003022448741486568922365}),
	FPC({ 0.940506070593268323787291309}, { 0.339776884406826857828825803}),
	FPC({-0.339776884406826857828825803}, { 0.940506070593268323787291309}),
	FPC({ 0.424779681209108833357226189}, { 0.905296759318118774354048329}),
	FPC({-0.905296759318118774354048329}, { 0.424779681209108833357226189}),
	FPC({ 0.738887324460615147933116508}, { 0.673829000378756060917568372}),
	FPC({-0.673829000378756060917568372}, { 0.738887324460615147933116508}),
	FPC({ 0.046003182130914628814301788}, { 0.998941293186856850633930266}),
	FPC({-0.998941293186856850633930266}, { 0.046003182130914628814301788}),
	FPC({ 0.999618822495178597116830637}, { 0.027608145778965741612354872}),
	FPC({-0.027608145778965741612354872}, { 0.999618822495178597116830637}),
	FPC({ 0.687315340891759108199186948}, { 0.726359155084345976817494315}),
	FPC({-0.726359155084345976817494315}, { 0.687315340891759108199186948}),
	FPC({ 0.912962190428398164628018233}, { 0.408044162864978680820747499}),
	FPC({-0.408044162864978680820747499}, { 0.912962190428398164628018233}),
	FPC({ 0.357030961233430032614954036}, { 0.934092550404258914729877883}),
	FPC({-0.934092550404258914729877883}, { 0.357030961233430032614954036}),
	FPC({ 0.975025345066994146844913468}, { 0.222093620973203534094094721}),
	FPC({-0.222093620973203534094094721}, { 0.975025345066994146844913468}),
	FPC({ 0.532403127877197971442805218}, { 0.846490938774052078300544488}),
	FPC({-0.846490938774052078300544488}, { 0.532403127877197971442805218}),
	FPC({ 0.815814410806733789010772660}, { 0.578313796411655563342245019}),
	FPC({-0.578313796411655563342245019}, { 0.815814410806733789010772660}),
	FPC({ 0.167938294974731178054745536}, { 0.985797509167567424700995000}),
	FPC({-0.985797509167567424700995000}, { 0.167938294974731178054745536}),
	FPC({ 0.992099313142191757112085445}, { 0.125454983411546238542336453}),
	FPC({-0.125454983411546238542336453}, { 0.992099313142191757112085445}),
	FPC({ 0.612810082429409703935211936}, { 0.790230221437310055030217152}),
	FPC({-0.790230221437310055030217152}, { 0.612810082429409703935211936}),
	FPC({ 0.868570705971340895340449876}, { 0.495565261825772531150266670}),
	FPC({-0.495565261825772531150266670}, { 0.868570705971340895340449876}),
	FPC({ 0.263754678974831383611349322}, { 0.964589793289812723836432159}),
	FPC({-0.964589793289812723836432159}, { 0.263754678974831383611349322}),
	FPC({ 0.948561349915730288158494826}, { 0.316593375556165867243047035}),
	FPC({-0.316593375556165867243047035}, { 0.948561349915730288158494826}),
	FPC({ 0.446868840162374195353044389}, { 0.894599485631382678433072126}),
	FPC({-0.894599485631382678433072126}, { 0.446868840162374195353044389}),
	FPC({ 0.755201376896536527598710756}, { 0.655492852999615385312679701}),
	FPC({-0.655492852999615385312679701}, { 0.755201376896536527598710756}),
	FPC({ 0.070504573389613863027351471}, { 0.997511456140303459699448390}),
	FPC({-0.997511456140303459699448390}, { 0.070504573389613863027351471}),
	FPC({ 0.997060070339482978987989949}, { 0.076623861392031492278332463}),
	FPC({-0.076623861392031492278332463}, { 0.997060070339482978987989949}),
	FPC({ 0.650846684996380915068975573}, { 0.759209188978388033485525443}),
	FPC({-0.759209188978388033485525443}, { 0.650846684996380915068975573}),
	FPC({ 0.891840709392342727796478697}, { 0.452349587233770874133026703}),
	FPC({-0.452349587233770874133026703}, { 0.891840709392342727796478697}),
	FPC({ 0.310767152749611495835997250}, { 0.950486073949481721759926101}),
	FPC({-0.950486073949481721759926101}, { 0.310767152749611495835997250}),
	FPC({ 0.962953266873683886347921481}, { 0.269668325572915106525464462}),
	FPC({-0.269668325572915106525464462}, { 0.962953266873683886347921481}),
	FPC({ 0.490226483288291154229598449}, { 0.871595086655951034842481435}),
	FPC({-0.871595086655951034842481435}, { 0.490226483288291154229598449}),
	FPC({ 0.786455213599085757522319464}, { 0.617647307937803932403979402}),
	FPC({-0.617647307937803932403979402}, { 0.786455213599085757522319464}),
	FPC({ 0.119365214810991364593637790}, { 0.992850414459865090793563344}),
	FPC({-0.992850414459865090793563344}, { 0.119365214810991364593637790}),
	FPC({ 0.984748501801904218556553176}, { 0.173983873387463827950700807}),
	FPC({-0.173983873387463827950700807}, { 0.984748501801904218556553176}),
	FPC({ 0.573297166698042212820171239}, { 0.819347520076796960824689637}),
	FPC({-0.819347520076796960824689637}, { 0.573297166698042212820171239}),
	FPC({ 0.843208239641845437161743865}, { 0.537587076295645482502214932}),
	FPC({-0.537587076295645482502214932}, { 0.843208239641845437161743865}),
	FPC({ 0.216106797076219509948385131}, { 0.976369731330021149312732194}),
	FPC({-0.976369731330021149312732194}, { 0.216106797076219509948385131}),
	FPC({ 0.931884265581668106718557199}, { 0.362755724367397216204854462}),
	FPC({-0.362755724367397216204854462}, { 0.931884265581668106718557199}),
	FPC({ 0.402434650859418441082533934}, { 0.915448716088267819566431292}),
	FPC({-0.915448716088267819566431292}, { 0.402434650859418441082533934}),
	FPC({ 0.722128193929215321243607198}, { 0.691759258364157774906734132}),
	FPC({-0.691759258364157774906734132}, { 0.722128193929215321243607198}),
	FPC({ 0.021474080275469507418374898}, { 0.999769405351215321657617036}),
	FPC({-0.999769405351215321657617036}, { 0.021474080275469507418374898}),
	FPC({ 0.999882347454212525633049627}, { 0.015339206284988101044151868}),
	FPC({-0.015339206284988101044151868}, { 0.999882347454212525633049627}),
	FPC({ 0.696177131491462944788582591}, { 0.717870045055731736211325329}),
	FPC({-0.717870045055731736211325329}, { 0.696177131491462944788582591}),
	FPC({ 0.917900775621390457642276297}, { 0.396809987416710328595290911}),
	FPC({-0.396809987416710328595290911}, { 0.917900775621390457642276297}),
	FPC({ 0.368466829953372331712746222}, { 0.929640895843181265457918066}),
	FPC({-0.929640895843181265457918066}, { 0.368466829953372331712746222}),
	FPC({ 0.977677357824509979943404762}, { 0.210111836880469621717489972}),
	FPC({-0.210111836880469621717489972}, { 0.977677357824509979943404762}),
	FPC({ 0.542750784864515906586768661}, { 0.839893794195999504583383987}),
	FPC({-0.839893794195999504583383987}, { 0.542750784864515906586768661}),
	FPC({ 0.822849781375826332046780034}, { 0.568258952670131549790548489}),
	FPC({-0.568258952670131549790548489}, { 0.822849781375826332046780034}),
	FPC({ 0.180022901405699522679906590}, { 0.983662419211730274396237776}),
	FPC({-0.983662419211730274396237776}, { 0.180022901405699522679906590}),
	FPC({ 0.993564135520595333782021697}, { 0.113270952177564349018228733}),
	FPC({-0.113270952177564349018228733}, { 0.993564135520595333782021697}),
	FPC({ 0.622461279374149972519166721}, { 0.782650596166575738458949301}),
	FPC({-0.782650596166575738458949301}, { 0.622461279374149972519166721}),
	FPC({ 0.874586652278176112634431897}, { 0.484869248000791101822951699}),
	FPC({-0.484869248000791101822951699}, { 0.874586652278176112634431897}),
	FPC({ 0.275571819310958163076425168}, { 0.961280485811320641748659653}),
	FPC({-0.961280485811320641748659653}, { 0.275571819310958163076425168}),
	FPC({ 0.952375012719765858529893608}, { 0.304929229735402406490728633}),
	FPC({-0.304929229735402406490728633}, { 0.952375012719765858529893608}),
	FPC({ 0.457813303598877221904961155}, { 0.889048355854664562540777729}),
	FPC({-0.889048355854664562540777729}, { 0.457813303598877221904961155}),
	FPC({ 0.763188417263381271704838297}, { 0.646176012983316364832802220}),
	FPC({-0.646176012983316364832802220}, { 0.763188417263381271704838297}),
	FPC({ 0.082740264549375693111987083}, { 0.996571145790554847093566910}),
	FPC({-0.996571145790554847093566910}, { 0.082740264549375693111987083}),
	FPC({ 0.997925286198596012623025462}, { 0.064382630929857460819324537}),
	FPC({-0.064382630929857460819324537}, { 0.997925286198596012623025462}),
	FPC({ 0.660114342067420478559490747}, { 0.751165131909686411205819422}),
	FPC({-0.751165131909686411205819422}, { 0.660114342067420478559490747}),
	FPC({ 0.897324580705418281231391836}, { 0.441371268731716692879988968}),
	FPC({-0.441371268731716692879988968}, { 0.897324580705418281231391836}),
	FPC({ 0.322407678801069848384807478}, { 0.946600913083283570044599823}),
	FPC({-0.946600913083283570044599823}, { 0.322407678801069848384807478}),
	FPC({ 0.966190003445412555433832961}, { 0.257831102162159005614471295}),
	FPC({-0.257831102162159005614471295}, { 0.966190003445412555433832961}),
	FPC({ 0.500885382611240786241285004}, { 0.865513624090569082825488358}),
	FPC({-0.865513624090569082825488358}, { 0.500885382611240786241285004}),
	FPC({ 0.793975477554337164895083757}, { 0.607949784967773667243642671}),
	FPC({-0.607949784967773667243642671}, { 0.793975477554337164895083757}),
	FPC({ 0.131540028702883111103387493}, { 0.991310859846115418957349799}),
	FPC({-0.991310859846115418957349799}, { 0.131540028702883111103387493}),
	FPC({ 0.986809401814185476970235952}, { 0.161886393780111837641387995}),
	FPC({-0.161886393780111837641387995}, { 0.986809401814185476970235952}),
	FPC({ 0.583308652937698294392830961}, { 0.812250586585203913049744181}),
	FPC({-0.812250586585203913049744181}, { 0.583308652937698294392830961}),
	FPC({ 0.849741768000852489471268395}, { 0.527199134781901348464274575}),
	FPC({-0.527199134781901348464274575}, { 0.849741768000852489471268395}),
	FPC({ 0.228072083170885739254457379}, { 0.973644249650811925318383912}),
	FPC({-0.973644249650811925318383912}, { 0.228072083170885739254457379}),
	FPC({ 0.936265667170278246576310996}, { 0.351292756085567125601307623}),
	FPC({-0.351292756085567125601307623}, { 0.936265667170278246576310996}),
	FPC({ 0.413638312238434547471944324}, { 0.910441292258067196934095369}),
	FPC({-0.910441292258067196934095369}, { 0.413638312238434547471944324}),
	FPC({ 0.730562769227827561177758850}, { 0.682845546385248068164596123}),
	FPC({-0.682845546385248068164596123}, { 0.730562769227827561177758850}),
	FPC({ 0.033741171851377584833716112}, { 0.999430604555461772019008327}),
	FPC({-0.999430604555461772019008327}, { 0.033741171851377584833716112}),
	FPC({ 0.999204758618363895492950001}, { 0.039872927587739811128578738}),
	FPC({-0.039872927587739811128578738}, { 0.999204758618363895492950001}),
	FPC({ 0.678350043129861486873655042}, { 0.734738878095963464563223604}),
	FPC({-0.734738878095963464563223604}, { 0.678350043129861486873655042}),
	FPC({ 0.907886116487666212038681480}, { 0.419216888363223956433010020}),
	FPC({-0.419216888363223956433010020}, { 0.907886116487666212038681480}),
	FPC({ 0.345541324963989065539191723}, { 0.938403534063108112192420774}),
	FPC({-0.938403534063108112192420774}, { 0.345541324963989065539191723}),
	FPC({ 0.972226497078936305708321144}, { 0.234041958583543423191242045}),
	FPC({-0.234041958583543423191242045}, { 0.972226497078936305708321144}),
	FPC({ 0.521975292937154342694258318}, { 0.852960604930363657746588082}),
	FPC({-0.852960604930363657746588082}, { 0.521975292937154342694258318}),
	FPC({ 0.808656181588174991946968128}, { 0.588281548222645304786439813}),
	FPC({-0.588281548222645304786439813}, { 0.808656181588174991946968128}),
	FPC({ 0.155828397654265235743101486}, { 0.987784141644572154230969032}),
	FPC({-0.987784141644572154230969032}, { 0.155828397654265235743101486}),
	FPC({ 0.990485084256457037998682243}, { 0.137620121586486044948441663}),
	FPC({-0.137620121586486044948441663}, { 0.990485084256457037998682243}),
	FPC({ 0.603066598540348201693430617}, { 0.797690840943391108362662755}),
	FPC({-0.797690840943391108362662755}, { 0.603066598540348201693430617}),
	FPC({ 0.862423956111040538690933878}, { 0.506186645345155291048942344}),
	FPC({-0.506186645345155291048942344}, { 0.862423956111040538690933878}),
	FPC({ 0.251897818154216950498106628}, { 0.967753837093475465243391912}),
	FPC({-0.967753837093475465243391912}, { 0.251897818154216950498106628}),
	FPC({ 0.944604837261480265659265493}, { 0.328209843579092526107916817}),
	FPC({-0.328209843579092526107916817}, { 0.944604837261480265659265493}),
	FPC({ 0.435857079922255491032544080}, { 0.900015892016160228714535267}),
	FPC({-0.900015892016160228714535267}, { 0.435857079922255491032544080}),
	FPC({ 0.747100605980180144323078847}, { 0.664710978203344868130324985}),
	FPC({-0.664710978203344868130324985}, { 0.747100605980180144323078847}),
	FPC({ 0.058258264500435759613979782}, { 0.998301544933892840738782163}),
	FPC({-0.998301544933892840738782163}, { 0.058258264500435759613979782}),
	FPC({ 0.996044700901251989887944810}, { 0.088853552582524596561586535}),
	FPC({-0.088853552582524596561586535}, { 0.996044700901251989887944810}),
	FPC({ 0.641481012808583151988739898}, { 0.767138911935820381181694573}),
	FPC({-0.767138911935820381181694573}, { 0.641481012808583151988739898}),
	FPC({ 0.886222530148880631647990821}, { 0.463259783551860197390719637}),
	FPC({-0.463259783551860197390719637}, { 0.886222530148880631647990821}),
	FPC({ 0.299079826308040476750336973}, { 0.954228095109105629780430732}),
	FPC({-0.954228095109105629780430732}, { 0.299079826308040476750336973}),
	FPC({ 0.959571513081984528335528181}, { 0.281464937925757984095231007}),
	FPC({-0.281464937925757984095231007}, { 0.959571513081984528335528181}),
	FPC({ 0.479493757660153026679839798}, { 0.877545290207261291668470750}),
	FPC({-0.877545290207261291668470750}, { 0.479493757660153026679839798}),
	FPC({ 0.778816512381475953374724325}, { 0.627251815495144113509622565}),
	FPC({-0.627251815495144113509622565}, { 0.778816512381475953374724325}),
	FPC({ 0.107172424956808849175529148}, { 0.994240449453187946358413442}),
	FPC({-0.994240449453187946358413442}, { 0.107172424956808849175529148}),
	FPC({ 0.982539302287441255907040396}, { 0.186055151663446648105438304}),
	FPC({-0.186055151663446648105438304}, { 0.982539302287441255907040396}),
	FPC({ 0.563199344013834115007363772}, { 0.826321062845663480311195452}),
	FPC({-0.826321062845663480311195452}, { 0.563199344013834115007363772}),
	FPC({ 0.836547727223511984524285790}, { 0.547894059173100165608820571}),
	FPC({-0.547894059173100165608820571}, { 0.836547727223511984524285790}),
	FPC({ 0.204108966092816874181696950}, { 0.978948175319062194715480124}),
	FPC({-0.978948175319062194715480124}, { 0.204108966092816874181696950}),
	FPC({ 0.927362525650401087274536959}, { 0.374164062971457997104393020}),
	FPC({-0.374164062971457997104393020}, { 0.927362525650401087274536959}),
	FPC({ 0.391170384302253888687512949}, { 0.920318276709110566440076541}),
	FPC({-0.920318276709110566440076541}, { 0.391170384302253888687512949}),
	FPC({ 0.713584868780793592903125099}, { 0.700568793943248366792866380}),
	FPC({-0.700568793943248366792866380}, { 0.713584868780793592903125099}),
	FPC({ 0.009203754782059819315102378}, { 0.999957644551963866333120920}),
	FPC({-0.999957644551963866333120920}, { 0.009203754782059819315102378}),
	FPC({ 0.999957644551963866333120920}, { 0.009203754782059819315102378}),
	FPC({-0.009203754782059819315102378}, { 0.999957644551963866333120920}),
	FPC({ 0.700568793943248366792866380}, { 0.713584868780793592903125099}),
	FPC({-0.713584868780793592903125099}, { 0.700568793943248366792866380}),
	FPC({ 0.920318276709110566440076541}, { 0.391170384302253888687512949}),
	FPC({-0.391170384302253888687512949}, { 0.920318276709110566440076541}),
	FPC({ 0.374164062971457997104393020}, { 0.927362525650401087274536959}),
	FPC({-0.927362525650401087274536959}, { 0.374164062971457997104393020}),
	FPC({ 0.978948175319062194715480124}, { 0.204108966092816874181696950}),
	FPC({-0.204108966092816874181696950}, { 0.978948175319062194715480124}),
	FPC({ 0.547894059173100165608820571}, { 0.836547727223511984524285790}),
	FPC({-0.836547727223511984524285790}, { 0.547894059173100165608820571}),
	FPC({ 0.826321062845663480311195452}, { 0.563199344013834115007363772}),
	FPC({-0.563199344013834115007363772}, { 0.826321062845663480311195452}),
	FPC({ 0.186055151663446648105438304}, { 0.982539302287441255907040396}),
	FPC({-0.982539302287441255907040396}, { 0.186055151663446648105438304}),
	FPC({ 0.994240449453187946358413442}, { 0.107172424956808849175529148}),
	FPC({-0.107172424956808849175529148}, { 0.994240449453187946358413442}),
	FPC({ 0.627251815495144113509622565}, { 0.778816512381475953374724325}),
	FPC({-0.778816512381475953374724325}, { 0.627251815495144113509622565}),
	FPC({ 0.877545290207261291668470750}, { 0.479493757660153026679839798}),
	FPC({-0.479493757660153026679839798}, { 0.877545290207261291668470750}),
	FPC({ 0.281464937925757984095231007}, { 0.959571513081984528335528181}),
	FPC({-0.959571513081984528335528181}, { 0.281464937925757984095231007}),
	FPC({ 0.954228095109105629780430732}, { 0.299079826308040476750336973}),
	FPC({-0.299079826308040476750336973}, { 0.954228095109105629780430732}),
	FPC({ 0.463259783551860197390719637}, { 0.886222530148880631647990821}),
	FPC({-0.886222530148880631647990821}, { 0.463259783551860197390719637}),
	FPC({ 0.767138911935820381181694573}, { 0.641481012808583151988739898}),
	FPC({-0.641481012808583151988739898}, { 0.767138911935820381181694573}),
	FPC({ 0.088853552582524596561586535}, { 0.996044700901251989887944810}),
	FPC({-0.996044700901251989887944810}, { 0.088853552582524596561586535}),
	FPC({ 0.998301544933892840738782163}, { 0.058258264500435759613979782}),
	FPC({-0.058258264500435759613979782}, { 0.998301544933892840738782163}),
	FPC({ 0.664710978203344868130324985}, { 0.747100605980180144323078847}),
	FPC({-0.747100605980180144323078847}, { 0.664710978203344868130324985}),
	FPC({ 0.900015892016160228714535267}, { 0.435857079922255491032544080}),
	FPC({-0.435857079922255491032544080}, { 0.900015892016160228714535267}),
	FPC({ 0.328209843579092526107916817}, { 0.944604837261480265659265493}),
	FPC({-0.944604837261480265659265493}, { 0.328209843579092526107916817}),
	FPC({ 0.967753837093475465243391912}, { 0.251897818154216950498106628}),
	FPC({-0.251897818154216950498106628}, { 0.967753837093475465243391912}),
	FPC({ 0.506186645345155291048942344}, { 0.862423956111040538690933878}),
	FPC({-0.862423956111040538690933878}, { 0.506186645345155291048942344}),
	FPC({ 0.797690840943391108362662755}, { 0.603066598540348201693430617}),
	FPC({-0.603066598540348201693430617}, { 0.797690840943391108362662755}),
	FPC({ 0.137620121586486044948441663}, { 0.990485084256457037998682243}),
	FPC({-0.990485084256457037998682243}, { 0.137620121586486044948441663}),
	FPC({ 0.987784141644572154230969032}, { 0.155828397654265235743101486}),
	FPC({-0.155828397654265235743101486}, { 0.987784141644572154230969032}),
	FPC({ 0.588281548222645304786439813}, { 0.808656181588174991946968128}),
	FPC({-0.808656181588174991946968128}, { 0.588281548222645304786439813}),
	FPC({ 0.852960604930363657746588082}, { 0.521975292937154342694258318}),
	FPC({-0.521975292937154342694258318}, { 0.852960604930363657746588082}),
	FPC({ 0.234041958583543423191242045}, { 0.972226497078936305708321144}),
	FPC({-0.972226497078936305708321144}, { 0.234041958583543423191242045}),
	FPC({ 0.938403534063108112192420774}, { 0.345541324963989065539191723}),
	FPC({-0.345541324963989065539191723}, { 0.938403534063108112192420774}),
	FPC({ 0.419216888363223956433010020}, { 0.907886116487666212038681480}),
	FPC({-0.907886116487666212038681480}, { 0.419216888363223956433010020}),
	FPC({ 0.734738878095963464563223604}, { 0.678350043129861486873655042}),
	FPC({-0.678350043129861486873655042}, { 0.734738878095963464563223604}),
	FPC({ 0.039872927587739811128578738}, { 0.999204758618363895492950001}),
	FPC({-0.999204758618363895492950001}, { 0.039872927587739811128578738}),
	FPC({ 0.999430604555461772019008327}, { 0.033741171851377584833716112}),
	FPC({-0.033741171851377584833716112}, { 0.999430604555461772019008327}),
	FPC({ 0.682845546385248068164596123}, { 0.730562769227827561177758850}),
	FPC({-0.730562769227827561177758850}, { 0.682845546385248068164596123}),
	FPC({ 0.910441292258067196934095369}, { 0.413638312238434547471944324}),
	FPC({-0.413638312238434547471944324}, { 0.910441292258067196934095369}),
	FPC({ 0.351292756085567125601307623}, { 0.936265667170278246576310996}),
	FPC({-0.936265667170278246576310996}, { 0.351292756085567125601307623}),
	FPC({ 0.973644249650811925318383912}, { 0.228072083170885739254457379}),
	FPC({-0.228072083170885739254457379}, { 0.973644249650811925318383912}),
	FPC({ 0.527199134781901348464274575}, { 0.849741768000852489471268395}),
	FPC({-0.849741768000852489471268395}, { 0.527199134781901348464274575}),
	FPC({ 0.812250586585203913049744181}, { 0.583308652937698294392830961}),
	FPC({-0.583308652937698294392830961}, { 0.812250586585203913049744181}),
	FPC({ 0.161886393780111837641387995}, { 0.986809401814185476970235952}),
	FPC({-0.986809401814185476970235952}, { 0.161886393780111837641387995}),
	FPC({ 0.991310859846115418957349799}, { 0.131540028702883111103387493}),
	FPC({-0.131540028702883111103387493}, { 0.991310859846115418957349799}),
	FPC({ 0.607949784967773667243642671}, { 0.793975477554337164895083757}),
	FPC({-0.793975477554337164895083757}, { 0.607949784967773667243642671}),
	FPC({ 0.865513624090569082825488358}, { 0.500885382611240786241285004}),
	FPC({-0.500885382611240786241285004}, { 0.865513624090569082825488358}),
	FPC({ 0.257831102162159005614471295}, { 0.966190003445412555433832961}),
	FPC({-0.966190003445412555433832961}, { 0.257831102162159005614471295}),
	FPC({ 0.946600913083283570044599823}, { 0.322407678801069848384807478}),
	FPC({-0.322407678801069848384807478}, { 0.946600913083283570044599823}),
	FPC({ 0.441371268731716692879988968}, { 0.897324580705418281231391836}),
	FPC({-0.897324580705418281231391836}, { 0.441371268731716692879988968}),
	FPC({ 0.751165131909686411205819422}, { 0.660114342067420478559490747}),
	FPC({-0.660114342067420478559490747}, { 0.751165131909686411205819422}),
	FPC({ 0.064382630929857460819324537}, { 0.997925286198596012623025462}),
	FPC({-0.997925286198596012623025462}, { 0.064382630929857460819324537}),
	FPC({ 0.996571145790554847093566910}, { 0.082740264549375693111987083}),
	FPC({-0.082740264549375693111987083}, { 0.996571145790554847093566910}),
	FPC({ 0.646176012983316364832802220}, { 0.763188417263381271704838297}),
	FPC({-0.763188417263381271704838297}, { 0.646176012983316364832802220}),
	FPC({ 0.889048355854664562540777729}, { 0.457813303598877221904961155}),
	FPC({-0.457813303598877221904961155}, { 0.889048355854664562540777729}),
	FPC({ 0.304929229735402406490728633}, { 0.952375012719765858529893608}),
	FPC({-0.952375012719765858529893608}, { 0.304929229735402406490728633}),
	FPC({ 0.961280485811320641748659653}, { 0.275571819310958163076425168}),
	FPC({-0.275571819310958163076425168}, { 0.961280485811320641748659653}),
	FPC({ 0.484869248000791101822951699}, { 0.874586652278176112634431897}),
	FPC({-0.874586652278176112634431897}, { 0.484869248000791101822951699}),
	FPC({ 0.782650596166575738458949301}, { 0.622461279374149972519166721}),
	FPC({-0.622461279374149972519166721}, { 0.782650596166575738458949301}),
	FPC({ 0.113270952177564349018228733}, { 0.993564135520595333782021697}),
	FPC({-0.993564135520595333782021697}, { 0.113270952177564349018228733}),
	FPC({ 0.983662419211730274396237776}, { 0.180022901405699522679906590}),
	FPC({-0.180022901405699522679906590}, { 0.983662419211730274396237776}),
	FPC({ 0.568258952670131549790548489}, { 0.822849781375826332046780034}),
	FPC({-0.822849781375826332046780034}, { 0.568258952670131549790548489}),
	FPC({ 0.839893794195999504583383987}, { 0.542750784864515906586768661}),
	FPC({-0.542750784864515906586768661}, { 0.839893794195999504583383987}),
	FPC({ 0.210111836880469621717489972}, { 0.977677357824509979943404762}),
	FPC({-0.977677357824509979943404762}, { 0.210111836880469621717489972}),
	FPC({ 0.929640895843181265457918066}, { 0.368466829953372331712746222}),
	FPC({-0.368466829953372331712746222}, { 0.929640895843181265457918066}),
	FPC({ 0.396809987416710328595290911}, { 0.917900775621390457642276297}),
	FPC({-0.917900775621390457642276297}, { 0.396809987416710328595290911}),
	FPC({ 0.717870045055731736211325329}, { 0.696177131491462944788582591}),
	FPC({-0.696177131491462944788582591}, { 0.717870045055731736211325329}),
	FPC({ 0.015339206284988101044151868}, { 0.999882347454212525633049627}),
	FPC({-0.999882347454212525633049627}, { 0.015339206284988101044151868}),
	FPC({ 0.999769405351215321657617036}, { 0.021474080275469507418374898}),
	FPC({-0.021474080275469507418374898}, { 0.999769405351215321657617036}),
	FPC({ 0.691759258364157774906734132}, { 0.722128193929215321243607198}),
	FPC({-0.722128193929215321243607198}, { 0.691759258364157774906734132}),
	FPC({ 0.915448716088267819566431292}, { 0.402434650859418441082533934}),
	FPC({-0.402434650859418441082533934}, { 0.915448716088267819566431292}),
	FPC({ 0.362755724367397216204854462}, { 0.931884265581668106718557199}),
	FPC({-0.931884265581668106718557199}, { 0.362755724367397216204854462}),
	FPC({ 0.976369731330021149312732194}, { 0.216106797076219509948385131}),
	FPC({-0.216106797076219509948385131}, { 0.976369731330021149312732194}),
	FPC({ 0.537587076295645482502214932}, { 0.843208239641845437161743865}),
	FPC({-0.843208239641845437161743865}, { 0.537587076295645482502214932}),
	FPC({ 0.819347520076796960824689637}, { 0.573297166698042212820171239}),
	FPC({-0.573297166698042212820171239}, { 0.819347520076796960824689637}),
	FPC({ 0.173983873387463827950700807}, { 0.984748501801904218556553176}),
	FPC({-0.984748501801904218556553176}, { 0.173983873387463827950700807}),
	FPC({ 0.992850414459865090793563344}, { 0.119365214810991364593637790}),
	FPC({-0.119365214810991364593637790}, { 0.992850414459865090793563344}),
	FPC({ 0.617647307937803932403979402}, { 0.786455213599085757522319464}),
	FPC({-0.786455213599085757522319464}, { 0.617647307937803932403979402}),
	FPC({ 0.871595086655951034842481435}, { 0.490226483288291154229598449}),
	FPC({-0.490226483288291154229598449}, { 0.871595086655951034842481435}),
	FPC({ 0.269668325572915106525464462}, { 0.962953266873683886347921481}),
	FPC({-0.962953266873683886347921481}, { 0.269668325572915106525464462}),
	FPC({ 0.950486073949481721759926101}, { 0.310767152749611495835997250}),
	FPC({-0.310767152749611495835997250}, { 0.950486073949481721759926101}),
	FPC({ 0.452349587233770874133026703}, { 0.891840709392342727796478697}),
	FPC({-0.891840709392342727796478697}, { 0.452349587233770874133026703}),
	FPC({ 0.759209188978388033485525443}, { 0.650846684996380915068975573}),
	FPC({-0.650846684996380915068975573}, { 0.759209188978388033485525443}),
	FPC({ 0.076623861392031492278332463}, { 0.997060070339482978987989949}),
	FPC({-0.997060070339482978987989949}, { 0.076623861392031492278332463}),
	FPC({ 0.997511456140303459699448390}, { 0.070504573389613863027351471}),
	FPC({-0.070504573389613863027351471}, { 0.997511456140303459699448390}),
	FPC({ 0.655492852999615385312679701}, { 0.755201376896536527598710756}),
	FPC({-0.755201376896536527598710756}, { 0.655492852999615385312679701}),
	FPC({ 0.894599485631382678433072126}, { 0.446868840162374195353044389}),
	FPC({-0.446868840162374195353044389}, { 0.894599485631382678433072126}),
	FPC({ 0.316593375556165867243047035}, { 0.948561349915730288158494826}),
	FPC({-0.948561349915730288158494826}, { 0.316593375556165867243047035}),
	FPC({ 0.964589793289812723836432159}, { 0.263754678974831383611349322}),
	FPC({-0.263754678974831383611349322}, { 0.964589793289812723836432159}),
	FPC({ 0.495565261825772531150266670}, { 0.868570705971340895340449876}),
	FPC({-0.868570705971340895340449876}, { 0.495565261825772531150266670}),
	FPC({ 0.790230221437310055030217152}, { 0.612810082429409703935211936}),
	FPC({-0.612810082429409703935211936}, { 0.790230221437310055030217152}),
	FPC({ 0.125454983411546238542336453}, { 0.992099313142191757112085445}),
	FPC({-0.992099313142191757112085445}, { 0.125454983411546238542336453}),
	FPC({ 0.985797509167567424700995000}, { 0.167938294974731178054745536}),
	FPC({-0.167938294974731178054745536}, { 0.985797509167567424700995000}),
	FPC({ 0.578313796411655563342245019}, { 0.815814410806733789010772660}),
	FPC({-0.815814410806733789010772660}, { 0.578313796411655563342245019}),
	FPC({ 0.846490938774052078300544488}, { 0.532403127877197971442805218}),
	FPC({-0.532403127877197971442805218}, { 0.846490938774052078300544488}),
	FPC({ 0.222093620973203534094094721}, { 0.975025345066994146844913468}),
	FPC({-0.975025345066994146844913468}, { 0.222093620973203534094094721}),
	FPC({ 0.934092550404258914729877883}, { 0.357030961233430032614954036}),
	FPC({-0.357030961233430032614954036}, { 0.934092550404258914729877883}),
	FPC({ 0.408044162864978680820747499}, { 0.912962190428398164628018233}),
	FPC({-0.912962190428398164628018233}, { 0.408044162864978680820747499}),
	FPC({ 0.726359155084345976817494315}, { 0.687315340891759108199186948}),
	FPC({-0.687315340891759108199186948}, { 0.726359155084345976817494315}),
	FPC({ 0.027608145778965741612354872}, { 0.999618822495178597116830637}),
	FPC({-0.999618822495178597116830637}, { 0.027608145778965741612354872}),
	FPC({ 0.998941293186856850633930266}, { 0.046003182130914628814301788}),
	FPC({-0.046003182130914628814301788}, { 0.998941293186856850633930266}),
	FPC({ 0.673829000378756060917568372}, { 0.738887324460615147933116508}),
	FPC({-0.738887324460615147933116508}, { 0.673829000378756060917568372}),
	FPC({ 0.905296759318118774354048329}, { 0.424779681209108833357226189}),
	FPC({-0.424779681209108833357226189}, { 0.905296759318118774354048329}),
	FPC({ 0.339776884406826857828825803}, { 0.940506070593268323787291309}),
	FPC({-0.940506070593268323787291309}, { 0.339776884406826857828825803}),
	FPC({ 0.970772140728950302138169611}, { 0.240003022448741486568922365}),
	FPC({-0.240003022448741486568922365}, { 0.970772140728950302138169611}),
	FPC({ 0.516731799017649881508753876}, { 0.856147328375194481019630732}),
	FPC({-0.856147328375194481019630732}, { 0.516731799017649881508753876}),
	FPC({ 0.805031331142963597922659282}, { 0.593232295039799808047809426}),
	FPC({-0.593232295039799808047809426}, { 0.805031331142963597922659282}),
	FPC({ 0.149764534677321517229695737}, { 0.988721691960323767604516485}),
	FPC({-0.988721691960323767604516485}, { 0.149764534677321517229695737}),
	FPC({ 0.989622017463200834623694454}, { 0.143695033150294454819773349}),
	FPC({-0.143695033150294454819773349}, { 0.989622017463200834623694454}),
	FPC({ 0.598160706996342311724958652}, { 0.801376171723140219430247777}),
	FPC({-0.801376171723140219430247777}, { 0.598160706996342311724958652}),
	FPC({ 0.859301818357008404783582139}, { 0.511468850437970399504391001}),
	FPC({-0.511468850437970399504391001}, { 0.859301818357008404783582139}),
	FPC({ 0.245955050335794611599924709}, { 0.969281235356548486048290738}),
	FPC({-0.969281235356548486048290738}, { 0.245955050335794611599924709}),
	FPC({ 0.942573197601446879280758735}, { 0.333999651442009404650865481}),
	FPC({-0.333999651442009404650865481}, { 0.942573197601446879280758735}),
	FPC({ 0.430326481340082633908199031}, { 0.902673318237258806751502391}),
	FPC({-0.902673318237258806751502391}, { 0.430326481340082633908199031}),
	FPC({ 0.743007952135121693517362293}, { 0.669282588346636065720696366}),
	FPC({-0.669282588346636065720696366}, { 0.743007952135121693517362293}),
	FPC({ 0.052131704680283321236358216}, { 0.998640218180265222418199049}),
	FPC({-0.998640218180265222418199049}, { 0.052131704680283321236358216}),
	FPC({ 0.995480755491926941769171600}, { 0.094963495329638998938034312}),
	FPC({-0.094963495329638998938034312}, { 0.995480755491926941769171600}),
	FPC({ 0.636761861236284230413943435}, { 0.771060524261813773200605759}),
	FPC({-0.771060524261813773200605759}, { 0.636761861236284230413943435}),
	FPC({ 0.883363338665731594736308015}, { 0.468688822035827933697617870}),
	FPC({-0.468688822035827933697617870}, { 0.883363338665731594736308015}),
	FPC({ 0.293219162694258650606608599}, { 0.956045251349996443270479823}),
	FPC({-0.956045251349996443270479823}, { 0.293219162694258650606608599}),
	FPC({ 0.957826413027532890321037029}, { 0.287347459544729526477331841}),
	FPC({-0.287347459544729526477331841}, { 0.957826413027532890321037029}),
	FPC({ 0.474100214650550014398580015}, { 0.880470889052160770806542929}),
	FPC({-0.880470889052160770806542929}, { 0.474100214650550014398580015}),
	FPC({ 0.774953106594873878359129282}, { 0.632018735939809021909403706}),
	FPC({-0.632018735939809021909403706}, { 0.774953106594873878359129282}),
	FPC({ 0.101069862754827824987887585}, { 0.994879330794805620591166107}),
	FPC({-0.994879330794805620591166107}, { 0.101069862754827824987887585}),
	FPC({ 0.981379193313754574318224190}, { 0.192080397049892441679288205}),
	FPC({-0.192080397049892441679288205}, { 0.981379193313754574318224190}),
	FPC({ 0.558118531220556115693702964}, { 0.829761233794523042469023765}),
	FPC({-0.829761233794523042469023765}, { 0.558118531220556115693702964}),
	FPC({ 0.833170164701913186439915922}, { 0.553016705580027531764226988}),
	FPC({-0.553016705580027531764226988}, { 0.833170164701913186439915922}),
	FPC({ 0.198098410717953586179324918}, { 0.980182135968117392690210009}),
	FPC({-0.980182135968117392690210009}, { 0.198098410717953586179324918}),
	FPC({ 0.925049240782677590302371869}, { 0.379847208924051170576281147}),
	FPC({-0.379847208924051170576281147}, { 0.925049240782677590302371869}),
	FPC({ 0.385516053843918864075607949}, { 0.922701128333878570437264227}),
	FPC({-0.922701128333878570437264227}, { 0.385516053843918864075607949}),
	FPC({ 0.709272826438865651316533772}, { 0.704934080375904908852523758}),
	FPC({-0.704934080375904908852523758}, { 0.709272826438865651316533772}),
	FPC({ 0.003067956762965976270145365}, { 0.999995293809576171511580126}),
	FPC({-0.999995293809576171511580126}, { 0.003067956762965976270145365})
};

static const fpr fpr_gm3_square[] = {
	FPC({  1.000000000000000000000000 }, {  0.000000000000000000000000 }),
	FPC({  0.500000000000000000000000 }, {  0.866025403784438646763723 }),
	FPC({  0.866025403784438646763723 }, {  0.500000000000000000000000 }),
	FPC({ -0.866025403784438646763723 }, {  0.500000000000000000000000 }),
	FPC({  0.965925826289068286749743 }, {  0.258819045102520762348899 }),
	FPC({ -0.258819045102520762348899 }, {  0.965925826289068286749743 }),
	FPC({  0.258819045102520762348899 }, {  0.965925826289068286749743 }),
	FPC({ -0.965925826289068286749743 }, {  0.258819045102520762348899 }),
	FPC({  0.991444861373810411144558 }, {  0.130526192220051591548406 }),
	FPC({ -0.130526192220051591548406 }, {  0.991444861373810411144558 }),
	FPC({  0.608761429008720639416098 }, {  0.793353340291235164579777 }),
	FPC({ -0.793353340291235164579777 }, {  0.608761429008720639416098 }),
	FPC({  0.793353340291235164579777 }, {  0.608761429008720639416098 }),
	FPC({ -0.608761429008720639416098 }, {  0.793353340291235164579777 }),
	FPC({  0.130526192220051591548406 }, {  0.991444861373810411144558 }),
	FPC({ -0.991444861373810411144558 }, {  0.130526192220051591548406 }),
	FPC({  0.997858923238603506738070 }, {  0.065403129230143066815316 }),
	FPC({ -0.065403129230143066815316 }, {  0.997858923238603506738070 }),
	FPC({  0.659345815100068868425125 }, {  0.751839807478977396407519 }),
	FPC({ -0.751839807478977396407519 }, {  0.659345815100068868425125 }),
	FPC({  0.896872741532688303894104 }, {  0.442288690219001281995239 }),
	FPC({ -0.442288690219001281995239 }, {  0.896872741532688303894104 }),
	FPC({  0.321439465303161580701058 }, {  0.946930129495105664255804 }),
	FPC({ -0.946930129495105664255804 }, {  0.321439465303161580701058 }),
	FPC({  0.946930129495105664255804 }, {  0.321439465303161580701058 }),
	FPC({ -0.321439465303161580701058 }, {  0.946930129495105664255804 }),
	FPC({  0.442288690219001281995239 }, {  0.896872741532688303894104 }),
	FPC({ -0.896872741532688303894104 }, {  0.442288690219001281995239 }),
	FPC({  0.751839807478977396407519 }, {  0.659345815100068868425125 }),
	FPC({ -0.659345815100068868425125 }, {  0.751839807478977396407519 }),
	FPC({  0.065403129230143066815316 }, {  0.997858923238603506738070 }),
	FPC({ -0.997858923238603506738070 }, {  0.065403129230143066815316 }),
	FPC({  0.999464587476365644429836 }, {  0.032719082821776142063660 }),
	FPC({ -0.032719082821776142063660 }, {  0.999464587476365644429836 }),
	FPC({  0.683592302022871280513498 }, {  0.729864072697835657350101 }),
	FPC({ -0.729864072697835657350101 }, {  0.683592302022871280513498 }),
	FPC({  0.910863824921175818573292 }, {  0.412707029804394737047702 }),
	FPC({ -0.412707029804394737047702 }, {  0.910863824921175818573292 }),
	FPC({  0.352250047921233506531752 }, {  0.935905926757325700291707 }),
	FPC({ -0.935905926757325700291707 }, {  0.352250047921233506531752 }),
	FPC({  0.973876979277333648149690 }, {  0.227076263034373207585697 }),
	FPC({ -0.227076263034373207585697 }, {  0.973876979277333648149690 }),
	FPC({  0.528067850650367995873449 }, {  0.849202181526578887649097 }),
	FPC({ -0.849202181526578887649097 }, {  0.528067850650367995873449 }),
	FPC({  0.812846684591615216579096 }, {  0.582477696867802149197135 }),
	FPC({ -0.582477696867802149197135 }, {  0.812846684591615216579096 }),
	FPC({  0.162895473394588739480801 }, {  0.986643332084879004746924 }),
	FPC({ -0.986643332084879004746924 }, {  0.162895473394588739480801 }),
	FPC({  0.986643332084879004746924 }, {  0.162895473394588739480801 }),
	FPC({ -0.162895473394588739480801 }, {  0.986643332084879004746924 }),
	FPC({  0.582477696867802149197135 }, {  0.812846684591615216579096 }),
	FPC({ -0.812846684591615216579096 }, {  0.582477696867802149197135 }),
	FPC({  0.849202181526578887649097 }, {  0.528067850650367995873449 }),
	FPC({ -0.528067850650367995873449 }, {  0.849202181526578887649097 }),
	FPC({  0.227076263034373207585697 }, {  0.973876979277333648149690 }),
	FPC({ -0.973876979277333648149690 }, {  0.227076263034373207585697 }),
	FPC({  0.935905926757325700291707 }, {  0.352250047921233506531752 }),
	FPC({ -0.352250047921233506531752 }, {  0.935905926757325700291707 }),
	FPC({  0.412707029804394737047702 }, {  0.910863824921175818573292 }),
	FPC({ -0.910863824921175818573292 }, {  0.412707029804394737047702 }),
	FPC({  0.729864072697835657350101 }, {  0.683592302022871280513498 }),
	FPC({ -0.683592302022871280513498 }, {  0.729864072697835657350101 }),
	FPC({  0.032719082821776142063660 }, {  0.999464587476365644429836 }),
	FPC({ -0.999464587476365644429836 }, {  0.032719082821776142063660 }),
	FPC({  0.999866137909561782862747 }, {  0.016361731626486781642972 }),
	FPC({ -0.016361731626486781642972 }, {  0.999866137909561782862747 }),
	FPC({  0.695442635009611651123516 }, {  0.718581617779698057196404 }),
	FPC({ -0.718581617779698057196404 }, {  0.695442635009611651123516 }),
	FPC({  0.917494496447491307924151 }, {  0.397748474527011052046815 }),
	FPC({ -0.397748474527011052046815 }, {  0.917494496447491307924151 }),
	FPC({  0.367515936594703565413932 }, {  0.930017223684012117056878 }),
	FPC({ -0.930017223684012117056878 }, {  0.367515936594703565413932 }),
	FPC({  0.977461974943571863388068 }, {  0.211111552358965165924454 }),
	FPC({ -0.211111552358965165924454 }, {  0.977461974943571863388068 }),
	FPC({  0.541891580574751716151236 }, {  0.840448401094438021017148 }),
	FPC({ -0.840448401094438021017148 }, {  0.541891580574751716151236 }),
	FPC({  0.822268218989775107838487 }, {  0.569100145878898230610767 }),
	FPC({ -0.569100145878898230610767 }, {  0.822268218989775107838487 }),
	FPC({  0.179016861276632682039992 }, {  0.983846005927077416087277 }),
	FPC({ -0.983846005927077416087277 }, {  0.179016861276632682039992 }),
	FPC({  0.993447779019444395513852 }, {  0.114286964966846398117475 }),
	FPC({ -0.114286964966846398117475 }, {  0.993447779019444395513852 }),
	FPC({  0.621660573370077408037741 }, {  0.783286749228650365398028 }),
	FPC({ -0.783286749228650365398028 }, {  0.621660573370077408037741 }),
	FPC({  0.874090341626758851545242 }, {  0.485763393716340056269053 }),
	FPC({ -0.485763393716340056269053 }, {  0.874090341626758851545242 }),
	FPC({  0.274588618184932341480468 }, {  0.961561797682961947144678 }),
	FPC({ -0.961561797682961947144678 }, {  0.274588618184932341480468 }),
	FPC({  0.952062677713924257100071 }, {  0.305903020096553462762691 }),
	FPC({ -0.305903020096553462762691 }, {  0.952062677713924257100071 }),
	FPC({  0.456903875630420676563536 }, {  0.889516075421856035272103 }),
	FPC({ -0.889516075421856035272103 }, {  0.456903875630420676563536 }),
	FPC({  0.762527203906388096372518 }, {  0.646956152534857365398024 }),
	FPC({ -0.646956152534857365398024 }, {  0.762527203906388096372518 }),
	FPC({  0.081721074133668223747713 }, {  0.996655239309180324931734 }),
	FPC({ -0.996655239309180324931734 }, {  0.081721074133668223747713 }),
	FPC({  0.996655239309180324931734 }, {  0.081721074133668223747713 }),
	FPC({ -0.081721074133668223747713 }, {  0.996655239309180324931734 }),
	FPC({  0.646956152534857365398024 }, {  0.762527203906388096372518 }),
	FPC({ -0.762527203906388096372518 }, {  0.646956152534857365398024 }),
	FPC({  0.889516075421856035272103 }, {  0.456903875630420676563536 }),
	FPC({ -0.456903875630420676563536 }, {  0.889516075421856035272103 }),
	FPC({  0.305903020096553462762691 }, {  0.952062677713924257100071 }),
	FPC({ -0.952062677713924257100071 }, {  0.305903020096553462762691 }),
	FPC({  0.961561797682961947144678 }, {  0.274588618184932341480468 }),
	FPC({ -0.274588618184932341480468 }, {  0.961561797682961947144678 }),
	FPC({  0.485763393716340056269053 }, {  0.874090341626758851545242 }),
	FPC({ -0.874090341626758851545242 }, {  0.485763393716340056269053 }),
	FPC({  0.783286749228650365398028 }, {  0.621660573370077408037741 }),
	FPC({ -0.621660573370077408037741 }, {  0.783286749228650365398028 }),
	FPC({  0.114286964966846398117475 }, {  0.993447779019444395513852 }),
	FPC({ -0.993447779019444395513852 }, {  0.114286964966846398117475 }),
	FPC({  0.983846005927077416087277 }, {  0.179016861276632682039992 }),
	FPC({ -0.179016861276632682039992 }, {  0.983846005927077416087277 }),
	FPC({  0.569100145878898230610767 }, {  0.822268218989775107838487 }),
	FPC({ -0.822268218989775107838487 }, {  0.569100145878898230610767 }),
	FPC({  0.840448401094438021017148 }, {  0.541891580574751716151236 }),
	FPC({ -0.541891580574751716151236 }, {  0.840448401094438021017148 }),
	FPC({  0.211111552358965165924454 }, {  0.977461974943571863388068 }),
	FPC({ -0.977461974943571863388068 }, {  0.211111552358965165924454 }),
	FPC({  0.930017223684012117056878 }, {  0.367515936594703565413932 }),
	FPC({ -0.367515936594703565413932 }, {  0.930017223684012117056878 }),
	FPC({  0.397748474527011052046815 }, {  0.917494496447491307924151 }),
	FPC({ -0.917494496447491307924151 }, {  0.397748474527011052046815 }),
	FPC({  0.718581617779698057196404 }, {  0.695442635009611651123516 }),
	FPC({ -0.695442635009611651123516 }, {  0.718581617779698057196404 }),
	FPC({  0.016361731626486781642972 }, {  0.999866137909561782862747 }),
	FPC({ -0.999866137909561782862747 }, {  0.016361731626486781642972 }),
	FPC({  0.999966533917401103457604 }, {  0.008181139603937129285199 }),
	FPC({ -0.008181139603937129285199 }, {  0.999966533917401103457604 }),
	FPC({  0.701298177800824325782608 }, {  0.712868056384379865292869 }),
	FPC({ -0.712868056384379865292869 }, {  0.701298177800824325782608 }),
	FPC({  0.920717827298247671744498 }, {  0.390229012882428021086708 }),
	FPC({ -0.390229012882428021086708 }, {  0.920717827298247671744498 }),
	FPC({  0.375112238017038003704784 }, {  0.926979400466832935219633 }),
	FPC({ -0.926979400466832935219633 }, {  0.375112238017038003704784 }),
	FPC({  0.979156396202233698078799 }, {  0.203107734407762881820280 }),
	FPC({ -0.203107734407762881820280 }, {  0.979156396202233698078799 }),
	FPC({  0.548749271285615899155350 }, {  0.835986983907946694786658 }),
	FPC({ -0.835986983907946694786658 }, {  0.548749271285615899155350 }),
	FPC({  0.826896588635696200443662 }, {  0.562354009234973105915763 }),
	FPC({ -0.562354009234973105915763 }, {  0.826896588635696200443662 }),
	FPC({  0.187059851806831979729800 }, {  0.982348518521815687340482 }),
	FPC({ -0.982348518521815687340482 }, {  0.187059851806831979729800 }),
	FPC({  0.994349529829318082779655 }, {  0.106155605260457482382245 }),
	FPC({ -0.106155605260457482382245 }, {  0.994349529829318082779655 }),
	FPC({  0.628047947071334209800931 }, {  0.778174643752597854435501 }),
	FPC({ -0.778174643752597854435501 }, {  0.628047947071334209800931 }),
	FPC({  0.878035187385662726821267 }, {  0.478596082007180812989115 }),
	FPC({ -0.478596082007180812989115 }, {  0.878035187385662726821267 }),
	FPC({  0.282446100084212509013816 }, {  0.959283170157393624701975 }),
	FPC({ -0.959283170157393624701975 }, {  0.282446100084212509013816 }),
	FPC({  0.954533451218388445389092 }, {  0.298103825042739842534610 }),
	FPC({ -0.298103825042739842534610 }, {  0.954533451218388445389092 }),
	FPC({  0.464165840040551545858019 }, {  0.885748312411290521960500 }),
	FPC({ -0.885748312411290521960500 }, {  0.464165840040551545858019 }),
	FPC({  0.767794523709511971549052 }, {  0.640696159939860775017278 }),
	FPC({ -0.640696159939860775017278 }, {  0.767794523709511971549052 }),
	FPC({  0.089872114899234968189008 }, {  0.995953313646547745670090 }),
	FPC({ -0.995953313646547745670090 }, {  0.089872114899234968189008 }),
	FPC({  0.998360600940222499201645 }, {  0.057237317287568625555758 }),
	FPC({ -0.057237317287568625555758 }, {  0.998360600940222499201645 }),
	FPC({  0.665474655803342221422320 }, {  0.746420446185273785375656 }),
	FPC({ -0.746420446185273785375656 }, {  0.665474655803342221422320 }),
	FPC({  0.900461152235363623973128 }, {  0.434936447443717110219834 }),
	FPC({ -0.434936447443717110219834 }, {  0.900461152235363623973128 }),
	FPC({  0.329175675568038923256279 }, {  0.944268698313316527305264 }),
	FPC({ -0.944268698313316527305264 }, {  0.329175675568038923256279 }),
	FPC({  0.968010935275722712107895 }, {  0.250908009411060001393596 }),
	FPC({ -0.250908009411060001393596 }, {  0.968010935275722712107895 }),
	FPC({  0.507068341687617066584577 }, {  0.861905851504774289367093 }),
	FPC({ -0.861905851504774289367093 }, {  0.507068341687617066584577 }),
	FPC({  0.798307152099031473245793 }, {  0.602250521716281977070059 }),
	FPC({ -0.602250521716281977070059 }, {  0.798307152099031473245793 }),
	FPC({  0.138632972840226673210469 }, {  0.990343828597664849189698 }),
	FPC({ -0.990343828597664849189698 }, {  0.138632972840226673210469 }),
	FPC({  0.987942983606322358576036 }, {  0.154818155082141033276159 }),
	FPC({ -0.154818155082141033276159 }, {  0.987942983606322358576036 }),
	FPC({  0.589108215824328211871173 }, {  0.808054150443073172252770 }),
	FPC({ -0.808054150443073172252770 }, {  0.589108215824328211871173 }),
	FPC({  0.853493958862750438789532 }, {  0.521102736689023407126650 }),
	FPC({ -0.521102736689023407126650 }, {  0.853493958862750438789532 }),
	FPC({  0.235036087205926703760905 }, {  0.971986644821279433630547 }),
	FPC({ -0.971986644821279433630547 }, {  0.235036087205926703760905 }),
	FPC({  0.938756412469813197912695 }, {  0.344581482449080469756741 }),
	FPC({ -0.344581482449080469756741 }, {  0.938756412469813197912695 }),
	FPC({  0.420145122228702444924117 }, {  0.907456928050818664693929 }),
	FPC({ -0.907456928050818664693929 }, {  0.420145122228702444924117 }),
	FPC({  0.735432211061518648161354 }, {  0.677598304995788701925786 }),
	FPC({ -0.677598304995788701925786 }, {  0.735432211061518648161354 }),
	FPC({  0.040894747161583448700929 }, {  0.999163459927648756521096 }),
	FPC({ -0.999163459927648756521096 }, {  0.040894747161583448700929 }),
	FPC({  0.999163459927648756521096 }, {  0.040894747161583448700929 }),
	FPC({ -0.040894747161583448700929 }, {  0.999163459927648756521096 }),
	FPC({  0.677598304995788701925786 }, {  0.735432211061518648161354 }),
	FPC({ -0.735432211061518648161354 }, {  0.677598304995788701925786 }),
	FPC({  0.907456928050818664693929 }, {  0.420145122228702444924117 }),
	FPC({ -0.420145122228702444924117 }, {  0.907456928050818664693929 }),
	FPC({  0.344581482449080469756741 }, {  0.938756412469813197912695 }),
	FPC({ -0.938756412469813197912695 }, {  0.344581482449080469756741 }),
	FPC({  0.971986644821279433630547 }, {  0.235036087205926703760905 }),
	FPC({ -0.235036087205926703760905 }, {  0.971986644821279433630547 }),
	FPC({  0.521102736689023407126650 }, {  0.853493958862750438789532 }),
	FPC({ -0.853493958862750438789532 }, {  0.521102736689023407126650 }),
	FPC({  0.808054150443073172252770 }, {  0.589108215824328211871173 }),
	FPC({ -0.589108215824328211871173 }, {  0.808054150443073172252770 }),
	FPC({  0.154818155082141033276159 }, {  0.987942983606322358576036 }),
	FPC({ -0.987942983606322358576036 }, {  0.154818155082141033276159 }),
	FPC({  0.990343828597664849189698 }, {  0.138632972840226673210469 }),
	FPC({ -0.138632972840226673210469 }, {  0.990343828597664849189698 }),
	FPC({  0.602250521716281977070059 }, {  0.798307152099031473245793 }),
	FPC({ -0.798307152099031473245793 }, {  0.602250521716281977070059 }),
	FPC({  0.861905851504774289367093 }, {  0.507068341687617066584577 }),
	FPC({ -0.507068341687617066584577 }, {  0.861905851504774289367093 }),
	FPC({  0.250908009411060001393596 }, {  0.968010935275722712107895 }),
	FPC({ -0.968010935275722712107895 }, {  0.250908009411060001393596 }),
	FPC({  0.944268698313316527305264 }, {  0.329175675568038923256279 }),
	FPC({ -0.329175675568038923256279 }, {  0.944268698313316527305264 }),
	FPC({  0.434936447443717110219834 }, {  0.900461152235363623973128 }),
	FPC({ -0.900461152235363623973128 }, {  0.434936447443717110219834 }),
	FPC({  0.746420446185273785375656 }, {  0.665474655803342221422320 }),
	FPC({ -0.665474655803342221422320 }, {  0.746420446185273785375656 }),
	FPC({  0.057237317287568625555758 }, {  0.998360600940222499201645 }),
	FPC({ -0.998360600940222499201645 }, {  0.057237317287568625555758 }),
	FPC({  0.995953313646547745670090 }, {  0.089872114899234968189008 }),
	FPC({ -0.089872114899234968189008 }, {  0.995953313646547745670090 }),
	FPC({  0.640696159939860775017278 }, {  0.767794523709511971549052 }),
	FPC({ -0.767794523709511971549052 }, {  0.640696159939860775017278 }),
	FPC({  0.885748312411290521960500 }, {  0.464165840040551545858019 }),
	FPC({ -0.464165840040551545858019 }, {  0.885748312411290521960500 }),
	FPC({  0.298103825042739842534610 }, {  0.954533451218388445389092 }),
	FPC({ -0.954533451218388445389092 }, {  0.298103825042739842534610 }),
	FPC({  0.959283170157393624701975 }, {  0.282446100084212509013816 }),
	FPC({ -0.282446100084212509013816 }, {  0.959283170157393624701975 }),
	FPC({  0.478596082007180812989115 }, {  0.878035187385662726821267 }),
	FPC({ -0.878035187385662726821267 }, {  0.478596082007180812989115 }),
	FPC({  0.778174643752597854435501 }, {  0.628047947071334209800931 }),
	FPC({ -0.628047947071334209800931 }, {  0.778174643752597854435501 }),
	FPC({  0.106155605260457482382245 }, {  0.994349529829318082779655 }),
	FPC({ -0.994349529829318082779655 }, {  0.106155605260457482382245 }),
	FPC({  0.982348518521815687340482 }, {  0.187059851806831979729800 }),
	FPC({ -0.187059851806831979729800 }, {  0.982348518521815687340482 }),
	FPC({  0.562354009234973105915763 }, {  0.826896588635696200443662 }),
	FPC({ -0.826896588635696200443662 }, {  0.562354009234973105915763 }),
	FPC({  0.835986983907946694786658 }, {  0.548749271285615899155350 }),
	FPC({ -0.548749271285615899155350 }, {  0.835986983907946694786658 }),
	FPC({  0.203107734407762881820280 }, {  0.979156396202233698078799 }),
	FPC({ -0.979156396202233698078799 }, {  0.203107734407762881820280 }),
	FPC({  0.926979400466832935219633 }, {  0.375112238017038003704784 }),
	FPC({ -0.375112238017038003704784 }, {  0.926979400466832935219633 }),
	FPC({  0.390229012882428021086708 }, {  0.920717827298247671744498 }),
	FPC({ -0.920717827298247671744498 }, {  0.390229012882428021086708 }),
	FPC({  0.712868056384379865292869 }, {  0.701298177800824325782608 }),
	FPC({ -0.701298177800824325782608 }, {  0.712868056384379865292869 }),
	FPC({  0.008181139603937129285199 }, {  0.999966533917401103457604 }),
	FPC({ -0.999966533917401103457604 }, {  0.008181139603937129285199 })
};

static const fpr fpr_gm3_cubic[] = {
	FPC({  1.000000000000000000000000 }, {  0.000000000000000000000000 }),
	FPC({  1.000000000000000000000000 }, {  0.000000000000000000000000 }),
	FPC({  0.939692620785908384054109 }, {  0.342020143325668733044100 }),
	FPC({ -0.173648177666930348851717 }, {  0.984807753012208059366743 }),
	FPC({  0.984807753012208059366743 }, {  0.173648177666930348851717 }),
	FPC({  0.342020143325668733044100 }, {  0.939692620785908384054109 }),
	FPC({  0.642787609686539326322643 }, {  0.766044443118978035202393 }),
	FPC({ -0.342020143325668733044100 }, {  0.939692620785908384054109 }),
	FPC({  0.996194698091745532295010 }, {  0.087155742747658173558064 }),
	FPC({  0.422618261740699436186978 }, {  0.906307787036649963242553 }),
	FPC({  0.819152044288991789684488 }, {  0.573576436351046096108032 }),
	FPC({ -0.087155742747658173558064 }, {  0.996194698091745532295010 }),
	FPC({  0.906307787036649963242553 }, {  0.422618261740699436186978 }),
	FPC({  0.087155742747658173558064 }, {  0.996194698091745532295010 }),
	FPC({  0.573576436351046096108032 }, {  0.819152044288991789684488 }),
	FPC({ -0.422618261740699436186978 }, {  0.906307787036649963242553 }),
	FPC({  0.999048221581857762403716 }, {  0.043619387365335999781753 }),
	FPC({  0.461748613235033930562931 }, {  0.887010833178221701054610 }),
	FPC({  0.843391445812885701272857 }, {  0.537299608346823831840786 }),
	FPC({ -0.043619387365335999781753 }, {  0.999048221581857762403716 }),
	FPC({  0.953716950748226921143847 }, {  0.300705799504273121622547 }),
	FPC({  0.216439613938102879759554 }, {  0.976296007119933365970886 }),
	FPC({  0.675590207615660244348339 }, {  0.737277336810124041384293 }),
	FPC({ -0.300705799504273121622547 }, {  0.953716950748226921143847 }),
	FPC({  0.976296007119933365970886 }, {  0.216439613938102879759554 }),
	FPC({  0.300705799504273121622547 }, {  0.953716950748226921143847 }),
	FPC({  0.737277336810124041384293 }, {  0.675590207615660244348339 }),
	FPC({ -0.216439613938102879759554 }, {  0.976296007119933365970886 }),
	FPC({  0.887010833178221701054610 }, {  0.461748613235033930562931 }),
	FPC({  0.043619387365335999781753 }, {  0.999048221581857762403716 }),
	FPC({  0.537299608346823831840786 }, {  0.843391445812885701272857 }),
	FPC({ -0.461748613235033930562931 }, {  0.887010833178221701054610 }),
	FPC({  0.999762027079909092304159 }, {  0.021814885034561121842893 }),
	FPC({  0.480988768919387642784138 }, {  0.876726755707507717395841 }),
	FPC({  0.854911870672946595552948 }, {  0.518773258160521449520021 }),
	FPC({ -0.021814885034561121842893 }, {  0.999762027079909092304159 }),
	FPC({  0.960049854385928674120442 }, {  0.279829014030992056746859 }),
	FPC({  0.237685892326173093534525 }, {  0.971342069813261434508048 }),
	FPC({  0.691513055782269377761189 }, {  0.722363962059755580585916 }),
	FPC({ -0.279829014030992056746859 }, {  0.960049854385928674120442 }),
	FPC({  0.988361510467760708504889 }, {  0.152123386189916701538605 }),
	FPC({  0.362438038283701645062109 }, {  0.932007869282798502644494 }),
	FPC({  0.779884483092881801105889 }, {  0.625923472184059063442779 }),
	FPC({ -0.152123386189916701538605 }, {  0.988361510467760708504889 }),
	FPC({  0.915311479119447142408176 }, {  0.402746689858737234818584 }),
	FPC({  0.108866874851964575967861 }, {  0.994056338222319612098292 }),
	FPC({  0.591309648363582377279708 }, {  0.806444604267482566440315 }),
	FPC({ -0.402746689858737234818584 }, {  0.915311479119447142408176 }),
	FPC({  0.994056338222319612098292 }, {  0.108866874851964575967861 }),
	FPC({  0.402746689858737234818584 }, {  0.915311479119447142408176 }),
	FPC({  0.806444604267482566440315 }, {  0.591309648363582377279708 }),
	FPC({ -0.108866874851964575967861 }, {  0.994056338222319612098292 }),
	FPC({  0.932007869282798502644494 }, {  0.362438038283701645062109 }),
	FPC({  0.152123386189916701538605 }, {  0.988361510467760708504889 }),
	FPC({  0.625923472184059063442779 }, {  0.779884483092881801105889 }),
	FPC({ -0.362438038283701645062109 }, {  0.932007869282798502644494 }),
	FPC({  0.971342069813261434508048 }, {  0.237685892326173093534525 }),
	FPC({  0.279829014030992056746859 }, {  0.960049854385928674120442 }),
	FPC({  0.722363962059755580585916 }, {  0.691513055782269377761189 }),
	FPC({ -0.237685892326173093534525 }, {  0.971342069813261434508048 }),
	FPC({  0.876726755707507717395841 }, {  0.480988768919387642784138 }),
	FPC({  0.021814885034561121842893 }, {  0.999762027079909092304159 }),
	FPC({  0.518773258160521449520021 }, {  0.854911870672946595552948 }),
	FPC({ -0.480988768919387642784138 }, {  0.876726755707507717395841 }),
	FPC({  0.999940505000149769486575 }, {  0.010908091494182373602502 }),
	FPC({  0.490523568159307993958213 }, {  0.871427925350261382567771 }),
	FPC({  0.860519833856079008965269 }, {  0.509416936840841775528362 }),
	FPC({ -0.010908091494182373602502 }, {  0.999940505000149769486575 }),
	FPC({  0.963045136707762673999708 }, {  0.269340053953226017808214 }),
	FPC({  0.248267239373716284335074 }, {  0.968691580356593095694484 }),
	FPC({  0.699351526403367077886270 }, {  0.714777897334046389664634 }),
	FPC({ -0.269340053953226017808214 }, {  0.963045136707762673999708 }),
	FPC({  0.989962083714807906208096 }, {  0.141333197823567492290986 }),
	FPC({  0.372582902144102970251539 }, {  0.927998912192184517483585 }),
	FPC({  0.786665714368617025192599 }, {  0.617379181570704935956557 }),
	FPC({ -0.141333197823567492290986 }, {  0.989962083714807906208096 }),
	FPC({  0.919650220405092214688153 }, {  0.392738427084574079775816 }),
	FPC({  0.119703655304962524548535 }, {  0.992809667009055022755909 }),
	FPC({  0.600071239924480942980093 }, {  0.799946565100129690139618 }),
	FPC({ -0.392738427084574079775816 }, {  0.919650220405092214688153 }),
	FPC({  0.997086132304466640681221 }, {  0.076283974503952307817548 }),
	FPC({  0.432479206330166200140499 }, {  0.901643907588816091436575 }),
	FPC({  0.825359933084863783619027 }, {  0.564606925974300440540721 }),
	FPC({ -0.076283974503952307817548 }, {  0.997086132304466640681221 }),
	FPC({  0.943367500789825227140600 }, {  0.331749541753352086596431 }),
	FPC({  0.184380219542663384577251 }, {  0.982854991665301179652666 }),
	FPC({  0.651105449911949093056236 }, {  0.758987281247161842563350 }),
	FPC({ -0.331749541753352086596431 }, {  0.943367500789825227140600 }),
	FPC({  0.978598865500938321624330 }, {  0.205777210692234889171456 }),
	FPC({  0.311091140771090935256564 }, {  0.950380082984561121226653 }),
	FPC({  0.744602872292326232055197 }, {  0.667507724729847386367766 }),
	FPC({ -0.205777210692234889171456 }, {  0.978598865500938321624330 }),
	FPC({  0.891994856589314190110398 }, {  0.452045546176719687587538 }),
	FPC({  0.054514501638006321875733 }, {  0.998512978939763109000868 }),
	FPC({  0.546467432763043421413330 }, {  0.837480354951307868234665 }),
	FPC({ -0.452045546176719687587538 }, {  0.891994856589314190110398 }),
	FPC({  0.998512978939763109000868 }, {  0.054514501638006321875733 }),
	FPC({  0.452045546176719687587538 }, {  0.891994856589314190110398 }),
	FPC({  0.837480354951307868234665 }, {  0.546467432763043421413330 }),
	FPC({ -0.054514501638006321875733 }, {  0.998512978939763109000868 }),
	FPC({  0.950380082984561121226653 }, {  0.311091140771090935256564 }),
	FPC({  0.205777210692234889171456 }, {  0.978598865500938321624330 }),
	FPC({  0.667507724729847386367766 }, {  0.744602872292326232055197 }),
	FPC({ -0.311091140771090935256564 }, {  0.950380082984561121226653 }),
	FPC({  0.982854991665301179652666 }, {  0.184380219542663384577251 }),
	FPC({  0.331749541753352086596431 }, {  0.943367500789825227140600 }),
	FPC({  0.758987281247161842563350 }, {  0.651105449911949093056236 }),
	FPC({ -0.184380219542663384577251 }, {  0.982854991665301179652666 }),
	FPC({  0.901643907588816091436575 }, {  0.432479206330166200140499 }),
	FPC({  0.076283974503952307817548 }, {  0.997086132304466640681221 }),
	FPC({  0.564606925974300440540721 }, {  0.825359933084863783619027 }),
	FPC({ -0.432479206330166200140499 }, {  0.901643907588816091436575 }),
	FPC({  0.992809667009055022755909 }, {  0.119703655304962524548535 }),
	FPC({  0.392738427084574079775816 }, {  0.919650220405092214688153 }),
	FPC({  0.799946565100129690139618 }, {  0.600071239924480942980093 }),
	FPC({ -0.119703655304962524548535 }, {  0.992809667009055022755909 }),
	FPC({  0.927998912192184517483585 }, {  0.372582902144102970251539 }),
	FPC({  0.141333197823567492290986 }, {  0.989962083714807906208096 }),
	FPC({  0.617379181570704935956557 }, {  0.786665714368617025192599 }),
	FPC({ -0.372582902144102970251539 }, {  0.927998912192184517483585 }),
	FPC({  0.968691580356593095694484 }, {  0.248267239373716284335074 }),
	FPC({  0.269340053953226017808214 }, {  0.963045136707762673999708 }),
	FPC({  0.714777897334046389664634 }, {  0.699351526403367077886270 }),
	FPC({ -0.248267239373716284335074 }, {  0.968691580356593095694484 }),
	FPC({  0.871427925350261382567771 }, {  0.490523568159307993958213 }),
	FPC({  0.010908091494182373602502 }, {  0.999940505000149769486575 }),
	FPC({  0.509416936840841775528362 }, {  0.860519833856079008965269 }),
	FPC({ -0.490523568159307993958213 }, {  0.871427925350261382567771 }),
	FPC({  0.999985126139421578118478 }, {  0.005454126871013843285541 }),
	FPC({  0.495269150643949468507324 }, {  0.868739586078832307152380 }),
	FPC({  0.863285459207818463866839 }, {  0.504715975495472109611154 }),
	FPC({ -0.005454126871013843285541 }, {  0.999985126139421578118478 }),
	FPC({  0.964499827334375163848955 }, {  0.264083477468698324313731 }),
	FPC({  0.253546913459559410027050 }, {  0.967323091151622768540370 }),
	FPC({  0.703239613682924444226639 }, {  0.710952913874815753821905 }),
	FPC({ -0.264083477468698324313731 }, {  0.964499827334375163848955 }),
	FPC({  0.990718208348812642836874 }, {  0.135931716851214172399546 }),
	FPC({  0.377638784201221584707773 }, {  0.925952994847483170465395 }),
	FPC({  0.790021277996268998065849 }, {  0.613079424147591058129101 }),
	FPC({ -0.135931716851214172399546 }, {  0.990718208348812642836874 }),
	FPC({  0.921778586864374680860720 }, {  0.387716696568920618178891 }),
	FPC({  0.125116784732119184012752 }, {  0.992142021173529645720514 }),
	FPC({  0.604425324604609027541623 }, {  0.796661802132255496847968 }),
	FPC({ -0.387716696568920618178891 }, {  0.921778586864374680860720 }),
	FPC({  0.997487364259519807328957 }, {  0.070844605599834112535112 }),
	FPC({  0.437390453959214263075867 }, {  0.899271700202643132179748 }),
	FPC({  0.828427094602809019644637 }, {  0.560096910300305544253089 }),
	FPC({ -0.070844605599834112535112 }, {  0.997487364259519807328957 }),
	FPC({  0.945162873363267751559437 }, {  0.326599361321622133088404 }),
	FPC({  0.189738092917336294329084 }, {  0.981834739707295271837762 }),
	FPC({  0.655235378385673138749358 }, {  0.755424780445931457230353 }),
	FPC({ -0.326599361321622133088404 }, {  0.945162873363267751559437 }),
	FPC({  0.979706644972129454293586 }, {  0.200436747622420506726035 }),
	FPC({  0.316270007193118384907261 }, {  0.948669216613496342593155 }),
	FPC({  0.748232468991075835867120 }, {  0.663436637779011069386325 }),
	FPC({ -0.200436747622420506726035 }, {  0.979706644972129454293586 }),
	FPC({  0.894447102942505187188661 }, {  0.447173769398160840413049 }),
	FPC({  0.059959707266400897781874 }, {  0.998200798188684835805126 }),
	FPC({  0.551027028790523995392077 }, {  0.834487395676104289406787 }),
	FPC({ -0.447173769398160840413049 }, {  0.894447102942505187188661 }),
	FPC({  0.999271267550625274326747 }, {  0.038169802825881608939302 }),
	FPC({  0.466579614870656109475453 }, {  0.884479204383658881437741 }),
	FPC({  0.846309401557777272498438 }, {  0.532691652679969164851294 }),
	FPC({ -0.038169802825881608939302 }, {  0.999271267550625274326747 }),
	FPC({  0.955342852876616171636493 }, {  0.295499633599718881598110 }),
	FPC({  0.221761236931957867995395 }, {  0.975100996714908524822793 }),
	FPC({  0.679601363115189643224684 }, {  0.733581615944658303641098 }),
	FPC({ -0.295499633599718881598110 }, {  0.955342852876616171636493 }),
	FPC({  0.985740204310908997617407 }, {  0.168274328419659385754866 }),
	FPC({  0.347140258939263739667021 }, {  0.937813222674739706765619 }),
	FPC({  0.769538894255080321010753 }, {  0.638599945371645257950386 }),
	FPC({ -0.168274328419659385754866 }, {  0.985740204310908997617407 }),
	FPC({  0.908599320358525552495228 }, {  0.417668858120910725713315 }),
	FPC({  0.092587818676915648425002 }, {  0.995704522352213980254615 }),
	FPC({  0.578035664231303254541301 }, {  0.816011501681609904070227 }),
	FPC({ -0.417668858120910725713315 }, {  0.908599320358525552495228 }),
	FPC({  0.994635326514431197946173 }, {  0.103443546195646440569417 }),
	FPC({  0.407732924394236658039289 }, {  0.913101233360750473356190 }),
	FPC({  0.809657687165104032786774 }, {  0.586902402120194539906885 }),
	FPC({ -0.103443546195646440569417 }, {  0.994635326514431197946173 }),
	FPC({  0.933970789771373479450748 }, {  0.357349358267000227689107 }),
	FPC({  0.157511772600397838918973 }, {  0.987517109468124891694502 }),
	FPC({  0.630167751201124664005395 }, {  0.776459017170975640531775 }),
	FPC({ -0.357349358267000227689107 }, {  0.933970789771373479450748 }),
	FPC({  0.972623991218938160043425 }, {  0.232384534135434462532817 }),
	FPC({  0.285061085601570783540623 }, {  0.958509351793530460031809 }),
	FPC({  0.726124817658095997498992 }, {  0.687562905617367376502802 }),
	FPC({ -0.232384534135434462532817 }, {  0.972623991218938160043425 }),
	FPC({  0.879337089165197052025731 }, {  0.476199835802657841795635 }),
	FPC({  0.027267389499518385314546 }, {  0.999628175608251627726712 }),
	FPC({  0.523428339805593785931076 }, {  0.852069699665678666711185 }),
	FPC({ -0.476199835802657841795635 }, {  0.879337089165197052025731 }),
	FPC({  0.999628175608251627726712 }, {  0.027267389499518385314546 }),
	FPC({  0.476199835802657841795635 }, {  0.879337089165197052025731 }),
	FPC({  0.852069699665678666711185 }, {  0.523428339805593785931076 }),
	FPC({ -0.027267389499518385314546 }, {  0.999628175608251627726712 }),
	FPC({  0.958509351793530460031809 }, {  0.285061085601570783540623 }),
	FPC({  0.232384534135434462532817 }, {  0.972623991218938160043425 }),
	FPC({  0.687562905617367376502802 }, {  0.726124817658095997498992 }),
	FPC({ -0.285061085601570783540623 }, {  0.958509351793530460031809 }),
	FPC({  0.987517109468124891694502 }, {  0.157511772600397838918973 }),
	FPC({  0.357349358267000227689107 }, {  0.933970789771373479450748 }),
	FPC({  0.776459017170975640531775 }, {  0.630167751201124664005395 }),
	FPC({ -0.157511772600397838918973 }, {  0.987517109468124891694502 }),
	FPC({  0.913101233360750473356190 }, {  0.407732924394236658039289 }),
	FPC({  0.103443546195646440569417 }, {  0.994635326514431197946173 }),
	FPC({  0.586902402120194539906885 }, {  0.809657687165104032786774 }),
	FPC({ -0.407732924394236658039289 }, {  0.913101233360750473356190 }),
	FPC({  0.995704522352213980254615 }, {  0.092587818676915648425002 }),
	FPC({  0.417668858120910725713315 }, {  0.908599320358525552495228 }),
	FPC({  0.816011501681609904070227 }, {  0.578035664231303254541301 }),
	FPC({ -0.092587818676915648425002 }, {  0.995704522352213980254615 }),
	FPC({  0.937813222674739706765619 }, {  0.347140258939263739667021 }),
	FPC({  0.168274328419659385754866 }, {  0.985740204310908997617407 }),
	FPC({  0.638599945371645257950386 }, {  0.769538894255080321010753 }),
	FPC({ -0.347140258939263739667021 }, {  0.937813222674739706765619 }),
	FPC({  0.975100996714908524822793 }, {  0.221761236931957867995395 }),
	FPC({  0.295499633599718881598110 }, {  0.955342852876616171636493 }),
	FPC({  0.733581615944658303641098 }, {  0.679601363115189643224684 }),
	FPC({ -0.221761236931957867995395 }, {  0.975100996714908524822793 }),
	FPC({  0.884479204383658881437741 }, {  0.466579614870656109475453 }),
	FPC({  0.038169802825881608939302 }, {  0.999271267550625274326747 }),
	FPC({  0.532691652679969164851294 }, {  0.846309401557777272498438 }),
	FPC({ -0.466579614870656109475453 }, {  0.884479204383658881437741 }),
	FPC({  0.998200798188684835805126 }, {  0.059959707266400897781874 }),
	FPC({  0.447173769398160840413049 }, {  0.894447102942505187188661 }),
	FPC({  0.834487395676104289406787 }, {  0.551027028790523995392077 }),
	FPC({ -0.059959707266400897781874 }, {  0.998200798188684835805126 }),
	FPC({  0.948669216613496342593155 }, {  0.316270007193118384907261 }),
	FPC({  0.200436747622420506726035 }, {  0.979706644972129454293586 }),
	FPC({  0.663436637779011069386325 }, {  0.748232468991075835867120 }),
	FPC({ -0.316270007193118384907261 }, {  0.948669216613496342593155 }),
	FPC({  0.981834739707295271837762 }, {  0.189738092917336294329084 }),
	FPC({  0.326599361321622133088404 }, {  0.945162873363267751559437 }),
	FPC({  0.755424780445931457230353 }, {  0.655235378385673138749358 }),
	FPC({ -0.189738092917336294329084 }, {  0.981834739707295271837762 }),
	FPC({  0.899271700202643132179748 }, {  0.437390453959214263075867 }),
	FPC({  0.070844605599834112535112 }, {  0.997487364259519807328957 }),
	FPC({  0.560096910300305544253089 }, {  0.828427094602809019644637 }),
	FPC({ -0.437390453959214263075867 }, {  0.899271700202643132179748 }),
	FPC({  0.992142021173529645720514 }, {  0.125116784732119184012752 }),
	FPC({  0.387716696568920618178891 }, {  0.921778586864374680860720 }),
	FPC({  0.796661802132255496847968 }, {  0.604425324604609027541623 }),
	FPC({ -0.125116784732119184012752 }, {  0.992142021173529645720514 }),
	FPC({  0.925952994847483170465395 }, {  0.377638784201221584707773 }),
	FPC({  0.135931716851214172399546 }, {  0.990718208348812642836874 }),
	FPC({  0.613079424147591058129101 }, {  0.790021277996268998065849 }),
	FPC({ -0.377638784201221584707773 }, {  0.925952994847483170465395 }),
	FPC({  0.967323091151622768540370 }, {  0.253546913459559410027050 }),
	FPC({  0.264083477468698324313731 }, {  0.964499827334375163848955 }),
	FPC({  0.710952913874815753821905 }, {  0.703239613682924444226639 }),
	FPC({ -0.253546913459559410027050 }, {  0.967323091151622768540370 }),
	FPC({  0.868739586078832307152380 }, {  0.495269150643949468507324 }),
	FPC({  0.005454126871013843285541 }, {  0.999985126139421578118478 }),
	FPC({  0.504715975495472109611154 }, {  0.863285459207818463866839 }),
	FPC({ -0.495269150643949468507324 }, {  0.868739586078832307152380 }),
	FPC({  0.999996281527941877306100 }, {  0.002727073576053814643613 }),
	FPC({  0.497636425769119060771003 }, {  0.867385720281199957226932 }),
	FPC({  0.864658646705146142583319 }, {  0.502359855758822816535097 }),
	FPC({ -0.002727073576053814643613 }, {  0.999996281527941877306100 }),
	FPC({  0.965216415941994446937616 }, {  0.261452233488434303844734 }),
	FPC({  0.256183931893832573411627 }, {  0.966628053099751576610947 }),
	FPC({  0.705175819611317272766213 }, {  0.709032484048161873525988 }),
	FPC({ -0.261452233488434303844734 }, {  0.965216415941994446937616 }),
	FPC({  0.991085220184009999609629 }, {  0.133229449945619823817351 }),
	FPC({  0.380162521906870934374187 }, {  0.924919702967456454925306 }),
	FPC({  0.791690253021836631107955 }, {  0.610922698277139065235442 }),
	FPC({ -0.133229449945619823817351 }, {  0.991085220184009999609629 }),
	FPC({  0.922832491214663627977550 }, {  0.385201496828007896938546 }),
	FPC({  0.127821963778486112452265 }, {  0.991797129243582446995999 }),
	FPC({  0.606595632415574550057453 }, {  0.795010527436177515525285 }),
	FPC({ -0.385201496828007896938546 }, {  0.922832491214663627977550 }),
	FPC({  0.997676853582564739566558 }, {  0.068124120732628018129402 }),
	FPC({  0.439841207626348342054201 }, {  0.898075560336542912353215 }),
	FPC({  0.829951439603914894223813 }, {  0.557835645956216397512357 }),
	FPC({ -0.068124120732628018129402 }, {  0.997676853582564739566558 }),
	FPC({  0.946050019289749023274936 }, {  0.324020618174006792581324 }),
	FPC({  0.192414922956246859723526 }, {  0.981313659042684264843640 }),
	FPC({  0.657293040868677472262316 }, {  0.753635096333502163551410 }),
	FPC({ -0.324020618174006792581324 }, {  0.946050019289749023274936 }),
	FPC({  0.980249607718456238521185 }, {  0.197764270200187221709880 }),
	FPC({  0.318855921904976153409794 }, {  0.947803197434007261127450 }),
	FPC({  0.750038927233820039417570 }, {  0.661393685813480085111392 }),
	FPC({ -0.197764270200187221709880 }, {  0.980249607718456238521185 }),
	FPC({  0.895663252736375533038260 }, {  0.444732883535381815855693 }),
	FPC({  0.062681651328241003352008 }, {  0.998033571873594251619162 }),
	FPC({  0.553300688338212435763470 }, {  0.832981601408134529686251 }),
	FPC({ -0.444732883535381815855693 }, {  0.895663252736375533038260 }),
	FPC({  0.999371643649028048439053 }, {  0.035444574623489224990868 }),
	FPC({  0.468989919774239100727887 }, {  0.883203518533612258461250 }),
	FPC({  0.847758943910123033470382 }, {  0.530381723874788947711166 }),
	FPC({ -0.035444574623489224990868 }, {  0.999371643649028048439053 }),
	FPC({  0.956145149703435200050101 }, {  0.292893244542436468564369 }),
	FPC({  0.224419584481119726339591 }, {  0.974492611619668241233019 }),
	FPC({  0.681599367077231772668650 }, {  0.731725565222315473710510 }),
	FPC({ -0.292893244542436468564369 }, {  0.956145149703435200050101 }),
	FPC({  0.986195435338064153919670 }, {  0.165585516632240889416176 }),
	FPC({  0.349696453766740779319446 }, {  0.936863058415137707587376 }),
	FPC({  0.771277541782896818171200 }, {  0.636498981571323374600224 }),
	FPC({ -0.165585516632240889416176 }, {  0.986195435338064153919670 }),
	FPC({  0.909734955463862875401721 }, {  0.415189487833162065265832 }),
	FPC({  0.095302833884162983378337 }, {  0.995448326058991188617158 }),
	FPC({  0.580258838225829123351327 }, {  0.814432121579699892023383 }),
	FPC({ -0.415189487833162065265832 }, {  0.909734955463862875401721 }),
	FPC({  0.994913726152204981803431 }, {  0.100730717826963155485823 }),
	FPC({  0.410221502496510371926232 }, {  0.911985920335125313437324 }),
	FPC({  0.811255202508162157951501 }, {  0.584692223655694609877199 }),
	FPC({ -0.100730717826963155485823 }, {  0.994913726152204981803431 }),
	FPC({  0.934941834819438335915815 }, {  0.354801022411804898770636 }),
	FPC({  0.160204218712404164651591 }, {  0.987083891220370474189014 }),
	FPC({  0.632282868808565575418378 }, {  0.774737616107034171264224 }),
	FPC({ -0.354801022411804898770636 }, {  0.934941834819438335915815 }),
	FPC({  0.973254104266328071668424 }, {  0.229731252834148376359926 }),
	FPC({  0.287673951135565722942757 }, {  0.957728405049183108284741 }),
	FPC({  0.727997152215034731924815 }, {  0.685580153130762348725667 }),
	FPC({ -0.229731252834148376359926 }, {  0.973254104266328071668424 }),
	FPC({  0.880632451363939913772032 }, {  0.473800048126567841006152 }),
	FPC({  0.029993347690072578782832 }, {  0.999550098341420012439434 }),
	FPC({  0.525750050214852171433283 }, {  0.850639103673867334989200 }),
	FPC({ -0.473800048126567841006152 }, {  0.880632451363939913772032 }),
	FPC({  0.999817800289289027000562 }, {  0.019088379310129051305154 }),
	FPC({  0.483377878744999477426777 }, {  0.875411803861465294113429 }),
	FPC({  0.856323424551336242808275 }, {  0.516439921544289549573785 }),
	FPC({ -0.019088379310129051305154 }, {  0.999817800289289027000562 }),
	FPC({  0.960809398777347845865341 }, {  0.277209846905032151945457 }),
	FPC({  0.240333929789721033640386 }, {  0.970690270988552476034211 }),
	FPC({  0.693480424083520324088754 }, {  0.720475468987626812224955 }),
	FPC({ -0.277209846905032151945457 }, {  0.960809398777347845865341 }),
	FPC({  0.988772686939879061816476 }, {  0.149427485964570501375156 }),
	FPC({  0.364978344600978823853553 }, {  0.931016008440418360468503 }),
	FPC({  0.781588522475847859093347 }, {  0.623794342338900237962923 }),
	FPC({ -0.149427485964570501375156 }, {  0.988772686939879061816476 }),
	FPC({  0.916406395415044460945779 }, {  0.400249070507859116486813 }),
	FPC({  0.111577334806607285212550 }, {  0.993755753873885382501892 }),
	FPC({  0.593506683366026266015079 }, {  0.804829060608437175733229 }),
	FPC({ -0.400249070507859116486813 }, {  0.916406395415044460945779 }),
	FPC({  0.996428673892644710662541 }, {  0.084438722423693154734038 }),
	FPC({  0.425088258264301357101884 }, {  0.905151905862116952199282 }),
	FPC({  0.820713183438423797465244 }, {  0.571340415628343353560656 }),
	FPC({ -0.084438722423693154734038 }, {  0.996428673892644710662541 }),
	FPC({  0.940621840660496337225766 }, {  0.339456260617564453095132 }),
	FPC({  0.176333175161766274528381 }, {  0.984330539675250476977656 }),
	FPC({  0.644874279057686023882524 }, {  0.764288665498730062697385 }),
	FPC({ -0.339456260617564453095132 }, {  0.940621840660496337225766 }),
	FPC({  0.976882623542492323027695 }, {  0.213776378070022562893929 }),
	FPC({  0.303305537633580056541241 }, {  0.952893357538399966380099 }),
	FPC({  0.739116979468377403486170 }, {  0.673577085908912266486454 }),
	FPC({ -0.213776378070022562893929 }, {  0.976882623542492323027695 }),
	FPC({  0.888266757295156029453623 }, {  0.459327952430883932515050 }),
	FPC({  0.046343703174142329998606 }, {  0.998925553370273429670792 }),
	FPC({  0.539597600939389497155742 }, {  0.841923054121013699455018 }),
	FPC({ -0.459327952430883932515050 }, {  0.888266757295156029453623 }),
	FPC({  0.998657931054079998074016 }, {  0.051791280567141811375037 }),
	FPC({  0.454476400861367861240543 }, {  0.890758778267212626829730 }),
	FPC({  0.838967497700070815454693 }, {  0.544181530192712136833473 }),
	FPC({ -0.051791280567141811375037 }, {  0.998657931054079998074016 }),
	FPC({  0.951224917452519226422142 }, {  0.308498227575861420226245 }),
	FPC({  0.208445156623090581518870 }, {  0.978034057014567993554995 }),
	FPC({  0.669535829438706573328750 }, {  0.742779760829428644903273 }),
	FPC({ -0.308498227575861420226245 }, {  0.951224917452519226422142 }),
	FPC({  0.983354155371139284293577 }, {  0.181699216053105909581375 }),
	FPC({  0.334320940735862640241168 }, {  0.942459287494949470968985 }),
	FPC({  0.760760071441843561387609 }, {  0.649033214635276644052408 }),
	FPC({ -0.181699216053105909581375 }, {  0.983354155371139284293577 }),
	FPC({  0.902819957466915067754028 }, {  0.430018748892926374456568 }),
	FPC({  0.079002818088581841124340 }, {  0.996874392656398033653368 }),
	FPC({  0.566855643763471659196800 }, {  0.823817139378333226629689 }),
	FPC({ -0.430018748892926374456568 }, {  0.902819957466915067754028 }),
	FPC({  0.993132415949388432820599 }, {  0.116995745181313837291317 }),
	FPC({  0.395244920512985608373467 }, {  0.918575774124641112207182 }),
	FPC({  0.801580028943327274915865 }, {  0.597887495436402824447132 }),
	FPC({ -0.116995745181313837291317 }, {  0.993132415949388432820599 }),
	FPC({  0.929011522441486189960616 }, {  0.370050795392946022580626 }),
	FPC({  0.144032371719814327245792 }, {  0.989572976539257174361796 }),
	FPC({  0.619522181146311151781170 }, {  0.784979150721671862714824 }),
	FPC({ -0.370050795392946022580626 }, {  0.929011522441486189960616 }),
	FPC({  0.969365021332314489869772 }, {  0.245624622986787450431955 }),
	FPC({  0.271965347364624132631605 }, {  0.962307045507222363782195 }),
	FPC({  0.716682422520434913350240 }, {  0.697399673967690357238168 }),
	FPC({ -0.245624622986787450431955 }, {  0.969365021332314489869772 }),
	FPC({  0.872762378831029180931839 }, {  0.488145296092467848984364 }),
	FPC({  0.013634962261560760793840 }, {  0.999907039581243179209939 }),
	FPC({  0.511761743488775330225574 }, {  0.859127416569468420137999 }),
	FPC({ -0.488145296092467848984364 }, {  0.872762378831029180931839 }),
	FPC({  0.999907039581243179209939 }, {  0.013634962261560760793840 }),
	FPC({  0.488145296092467848984364 }, {  0.872762378831029180931839 }),
	FPC({  0.859127416569468420137999 }, {  0.511761743488775330225574 }),
	FPC({ -0.013634962261560760793840 }, {  0.999907039581243179209939 }),
	FPC({  0.962307045507222363782195 }, {  0.271965347364624132631605 }),
	FPC({  0.245624622986787450431955 }, {  0.969365021332314489869772 }),
	FPC({  0.697399673967690357238168 }, {  0.716682422520434913350240 }),
	FPC({ -0.271965347364624132631605 }, {  0.962307045507222363782195 }),
	FPC({  0.989572976539257174361796 }, {  0.144032371719814327245792 }),
	FPC({  0.370050795392946022580626 }, {  0.929011522441486189960616 }),
	FPC({  0.784979150721671862714824 }, {  0.619522181146311151781170 }),
	FPC({ -0.144032371719814327245792 }, {  0.989572976539257174361796 }),
	FPC({  0.918575774124641112207182 }, {  0.395244920512985608373467 }),
	FPC({  0.116995745181313837291317 }, {  0.993132415949388432820599 }),
	FPC({  0.597887495436402824447132 }, {  0.801580028943327274915865 }),
	FPC({ -0.395244920512985608373467 }, {  0.918575774124641112207182 }),
	FPC({  0.996874392656398033653368 }, {  0.079002818088581841124340 }),
	FPC({  0.430018748892926374456568 }, {  0.902819957466915067754028 }),
	FPC({  0.823817139378333226629689 }, {  0.566855643763471659196800 }),
	FPC({ -0.079002818088581841124340 }, {  0.996874392656398033653368 }),
	FPC({  0.942459287494949470968985 }, {  0.334320940735862640241168 }),
	FPC({  0.181699216053105909581375 }, {  0.983354155371139284293577 }),
	FPC({  0.649033214635276644052408 }, {  0.760760071441843561387609 }),
	FPC({ -0.334320940735862640241168 }, {  0.942459287494949470968985 }),
	FPC({  0.978034057014567993554995 }, {  0.208445156623090581518870 }),
	FPC({  0.308498227575861420226245 }, {  0.951224917452519226422142 }),
	FPC({  0.742779760829428644903273 }, {  0.669535829438706573328750 }),
	FPC({ -0.208445156623090581518870 }, {  0.978034057014567993554995 }),
	FPC({  0.890758778267212626829730 }, {  0.454476400861367861240543 }),
	FPC({  0.051791280567141811375037 }, {  0.998657931054079998074016 }),
	FPC({  0.544181530192712136833473 }, {  0.838967497700070815454693 }),
	FPC({ -0.454476400861367861240543 }, {  0.890758778267212626829730 }),
	FPC({  0.998925553370273429670792 }, {  0.046343703174142329998606 }),
	FPC({  0.459327952430883932515050 }, {  0.888266757295156029453623 }),
	FPC({  0.841923054121013699455018 }, {  0.539597600939389497155742 }),
	FPC({ -0.046343703174142329998606 }, {  0.998925553370273429670792 }),
	FPC({  0.952893357538399966380099 }, {  0.303305537633580056541241 }),
	FPC({  0.213776378070022562893929 }, {  0.976882623542492323027695 }),
	FPC({  0.673577085908912266486454 }, {  0.739116979468377403486170 }),
	FPC({ -0.303305537633580056541241 }, {  0.952893357538399966380099 }),
	FPC({  0.984330539675250476977656 }, {  0.176333175161766274528381 }),
	FPC({  0.339456260617564453095132 }, {  0.940621840660496337225766 }),
	FPC({  0.764288665498730062697385 }, {  0.644874279057686023882524 }),
	FPC({ -0.176333175161766274528381 }, {  0.984330539675250476977656 }),
	FPC({  0.905151905862116952199282 }, {  0.425088258264301357101884 }),
	FPC({  0.084438722423693154734038 }, {  0.996428673892644710662541 }),
	FPC({  0.571340415628343353560656 }, {  0.820713183438423797465244 }),
	FPC({ -0.425088258264301357101884 }, {  0.905151905862116952199282 }),
	FPC({  0.993755753873885382501892 }, {  0.111577334806607285212550 }),
	FPC({  0.400249070507859116486813 }, {  0.916406395415044460945779 }),
	FPC({  0.804829060608437175733229 }, {  0.593506683366026266015079 }),
	FPC({ -0.111577334806607285212550 }, {  0.993755753873885382501892 }),
	FPC({  0.931016008440418360468503 }, {  0.364978344600978823853553 }),
	FPC({  0.149427485964570501375156 }, {  0.988772686939879061816476 }),
	FPC({  0.623794342338900237962923 }, {  0.781588522475847859093347 }),
	FPC({ -0.364978344600978823853553 }, {  0.931016008440418360468503 }),
	FPC({  0.970690270988552476034211 }, {  0.240333929789721033640386 }),
	FPC({  0.277209846905032151945457 }, {  0.960809398777347845865341 }),
	FPC({  0.720475468987626812224955 }, {  0.693480424083520324088754 }),
	FPC({ -0.240333929789721033640386 }, {  0.970690270988552476034211 }),
	FPC({  0.875411803861465294113429 }, {  0.483377878744999477426777 }),
	FPC({  0.019088379310129051305154 }, {  0.999817800289289027000562 }),
	FPC({  0.516439921544289549573785 }, {  0.856323424551336242808275 }),
	FPC({ -0.483377878744999477426777 }, {  0.875411803861465294113429 }),
	FPC({  0.999550098341420012439434 }, {  0.029993347690072578782832 }),
	FPC({  0.473800048126567841006152 }, {  0.880632451363939913772032 }),
	FPC({  0.850639103673867334989200 }, {  0.525750050214852171433283 }),
	FPC({ -0.029993347690072578782832 }, {  0.999550098341420012439434 }),
	FPC({  0.957728405049183108284741 }, {  0.287673951135565722942757 }),
	FPC({  0.229731252834148376359926 }, {  0.973254104266328071668424 }),
	FPC({  0.685580153130762348725667 }, {  0.727997152215034731924815 }),
	FPC({ -0.287673951135565722942757 }, {  0.957728405049183108284741 }),
	FPC({  0.987083891220370474189014 }, {  0.160204218712404164651591 }),
	FPC({  0.354801022411804898770636 }, {  0.934941834819438335915815 }),
	FPC({  0.774737616107034171264224 }, {  0.632282868808565575418378 }),
	FPC({ -0.160204218712404164651591 }, {  0.987083891220370474189014 }),
	FPC({  0.911985920335125313437324 }, {  0.410221502496510371926232 }),
	FPC({  0.100730717826963155485823 }, {  0.994913726152204981803431 }),
	FPC({  0.584692223655694609877199 }, {  0.811255202508162157951501 }),
	FPC({ -0.410221502496510371926232 }, {  0.911985920335125313437324 }),
	FPC({  0.995448326058991188617158 }, {  0.095302833884162983378337 }),
	FPC({  0.415189487833162065265832 }, {  0.909734955463862875401721 }),
	FPC({  0.814432121579699892023383 }, {  0.580258838225829123351327 }),
	FPC({ -0.095302833884162983378337 }, {  0.995448326058991188617158 }),
	FPC({  0.936863058415137707587376 }, {  0.349696453766740779319446 }),
	FPC({  0.165585516632240889416176 }, {  0.986195435338064153919670 }),
	FPC({  0.636498981571323374600224 }, {  0.771277541782896818171200 }),
	FPC({ -0.349696453766740779319446 }, {  0.936863058415137707587376 }),
	FPC({  0.974492611619668241233019 }, {  0.224419584481119726339591 }),
	FPC({  0.292893244542436468564369 }, {  0.956145149703435200050101 }),
	FPC({  0.731725565222315473710510 }, {  0.681599367077231772668650 }),
	FPC({ -0.224419584481119726339591 }, {  0.974492611619668241233019 }),
	FPC({  0.883203518533612258461250 }, {  0.468989919774239100727887 }),
	FPC({  0.035444574623489224990868 }, {  0.999371643649028048439053 }),
	FPC({  0.530381723874788947711166 }, {  0.847758943910123033470382 }),
	FPC({ -0.468989919774239100727887 }, {  0.883203518533612258461250 }),
	FPC({  0.998033571873594251619162 }, {  0.062681651328241003352008 }),
	FPC({  0.444732883535381815855693 }, {  0.895663252736375533038260 }),
	FPC({  0.832981601408134529686251 }, {  0.553300688338212435763470 }),
	FPC({ -0.062681651328241003352008 }, {  0.998033571873594251619162 }),
	FPC({  0.947803197434007261127450 }, {  0.318855921904976153409794 }),
	FPC({  0.197764270200187221709880 }, {  0.980249607718456238521185 }),
	FPC({  0.661393685813480085111392 }, {  0.750038927233820039417570 }),
	FPC({ -0.318855921904976153409794 }, {  0.947803197434007261127450 }),
	FPC({  0.981313659042684264843640 }, {  0.192414922956246859723526 }),
	FPC({  0.324020618174006792581324 }, {  0.946050019289749023274936 }),
	FPC({  0.753635096333502163551410 }, {  0.657293040868677472262316 }),
	FPC({ -0.192414922956246859723526 }, {  0.981313659042684264843640 }),
	FPC({  0.898075560336542912353215 }, {  0.439841207626348342054201 }),
	FPC({  0.068124120732628018129402 }, {  0.997676853582564739566558 }),
	FPC({  0.557835645956216397512357 }, {  0.829951439603914894223813 }),
	FPC({ -0.439841207626348342054201 }, {  0.898075560336542912353215 }),
	FPC({  0.991797129243582446995999 }, {  0.127821963778486112452265 }),
	FPC({  0.385201496828007896938546 }, {  0.922832491214663627977550 }),
	FPC({  0.795010527436177515525285 }, {  0.606595632415574550057453 }),
	FPC({ -0.127821963778486112452265 }, {  0.991797129243582446995999 }),
	FPC({  0.924919702967456454925306 }, {  0.380162521906870934374187 }),
	FPC({  0.133229449945619823817351 }, {  0.991085220184009999609629 }),
	FPC({  0.610922698277139065235442 }, {  0.791690253021836631107955 }),
	FPC({ -0.380162521906870934374187 }, {  0.924919702967456454925306 }),
	FPC({  0.966628053099751576610947 }, {  0.256183931893832573411627 }),
	FPC({  0.261452233488434303844734 }, {  0.965216415941994446937616 }),
	FPC({  0.709032484048161873525988 }, {  0.705175819611317272766213 }),
	FPC({ -0.256183931893832573411627 }, {  0.966628053099751576610947 }),
	FPC({  0.867385720281199957226932 }, {  0.497636425769119060771003 }),
	FPC({  0.002727073576053814643613 }, {  0.999996281527941877306100 }),
	FPC({  0.502359855758822816535097 }, {  0.864658646705146142583319 }),
	FPC({ -0.497636425769119060771003 }, {  0.867385720281199957226932 })
};

#endif
